/*********************************************************************************

 R[hNX


 Copyright (c) 2006-2013, Isao MORI (Orios)
 All rights reserved.

 Redistribution and use in source and binary forms, with or without modification,
 are permitted provided that the following conditions are met:
   * Redistributions of source code must retain the above copyright notice, this
     list of conditions and the following disclaimer.
   * Redistributions in binary form must reproduce the above copyright notice,
     this list of conditions and the following disclaimer in the documentation
     and/or other materials provided with the distribution.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*********************************************************************************/

//==================================================
// CN[h
//==================================================
#include "CharSet.h"

//==================================================
// R[hNX{
//==================================================
int CharSet::nBufSize = 0;
unsigned char* CharSet::pucText = NULL;
unsigned char* CharSet::pucTempText = NULL;

//----------------------------------------
// ϊ֐
//----------------------------------------
int CharSet::ConvertText(unsigned char* pucOriginalText, int nType)
{
	int i = 0;
	int j;

	if (pucText != NULL) {
		free(pucText);
	}
	nBufSize = 0;

	switch(nType) {
	case SHIFTJIS_TO_UCS2:
		nBufSize = MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, (LPCSTR)pucOriginalText, -1, NULL, 0);
		nBufSize *= 2;
		pucText = (unsigned char*)malloc(nBufSize);
		MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, (LPCSTR)pucOriginalText, -1, (wchar_t*)pucText, nBufSize);
		return nBufSize;
		
	case UCS2_TO_UTF8:
		
		// Kvȃobt@̌vZ
		while ((pucOriginalText[i + 1] != 0) || (pucOriginalText[i] != 0)) {
			if ((pucOriginalText[i + 1] == 0) && (pucOriginalText[i] < 0x80)) {
				nBufSize++;
			}
			else if ((pucOriginalText[i + 1] < 0x08)) {
				nBufSize += 2;
			}
			else {
				nBufSize += 3;
			}
			i += 2;
		}
		nBufSize++;
		
		// UTF-8ւ̕ϊ
		i = 0;
		j = 0;
		pucText = (unsigned char*)malloc(nBufSize);
		while ((pucOriginalText[i + 1] != 0) || (pucOriginalText[i] != 0)) {
			if ((pucOriginalText[i + 1] == 0) && (pucOriginalText[i] < 0x80)) {
				pucText[j] = pucOriginalText[i];
				j++;
			}
			else if ((pucOriginalText[i + 1] < 0x08)) {
				pucText[j + 1] = 0x80 | (pucOriginalText[i] & 0x3F);
				pucText[j] = 0xC0 | (pucOriginalText[i + 1] << 2) | (pucOriginalText[i] >> 6);
				j += 2;
			}
			else {
				pucText[j + 2] = 0x80 | (pucOriginalText[i] & 0x3F);
				pucText[j + 1] = 0x80 | ((pucOriginalText[i + 1] << 2) & 0x3C) | (pucOriginalText[i] >> 6);
				pucText[j] = 0xE0 | (pucOriginalText[i + 1] >> 4);
				j += 3;
			}
			i += 2;
		}
		pucText[nBufSize - 1] = NULL;
		
		return nBufSize;
		
	case SHIFTJIS_TO_UTF8:
		CharSet::ConvertText((unsigned char*)pucOriginalText, SHIFTJIS_TO_UCS2);
		pucTempText = (unsigned char*)malloc(nBufSize);
		CharSet::CopyText((unsigned char*)pucTempText);
		CharSet::ConvertText((unsigned char*)pucTempText, UCS2_TO_UTF8);
		free(pucTempText);
		return nBufSize;
	
	case UTF8_TO_UCS2:

		// Kvȃobt@̌vZ
		i = 0;
		while (pucOriginalText[i] != 0) {
			if (pucOriginalText[i] < 0x80) {
				i++;
			}
			else if (((pucOriginalText[i] & 0xE0) == 0xC0)) {
				i += 2;
			}
			else if (((pucOriginalText[i] & 0xF0) == 0xE0)) {
				i += 3;
			}
			else {
				nBufSize = -1;
				return 0;
			}
			nBufSize += 2;
		}
		nBufSize += 2;
		
		// UTF-8ւ̕ϊ
		i = 0;
		j = 0;
		pucText = (unsigned char*)malloc(nBufSize);
		while (pucOriginalText[i] != 0) {
			if (pucOriginalText[i] < 0x80) {
				pucText[j] = pucOriginalText[i];
				pucText[j + 1] = 0;
				i++;
			}
			else if (((pucOriginalText[i] & 0xE0) == 0xC0)) {
				pucText[j] = ((pucOriginalText[i] & 0x03) << 6) | (pucOriginalText[i + 1] & 0x3F);
				pucText[j + 1] = (pucOriginalText[i] & 0x1C) >> 2;
				i += 2;
			}
			else if (((pucOriginalText[i] & 0xF0) == 0xE0)) {
				pucText[j] = ((pucOriginalText[i + 1] & 0x03) << 6) | (pucOriginalText[i + 2] & 0x3F);
				pucText[j + 1] = ((pucOriginalText[i] & 0x0F) << 4) | ((pucOriginalText[i + 1] & 0x3C) >> 2);
				i += 3;
			}
			j += 2;
		}
		pucText[nBufSize - 1] = pucText[nBufSize - 2] = NULL;
		
		return nBufSize;

	case UCS2_TO_SHIFTJIS:
		nBufSize = WideCharToMultiByte(CP_ACP, WC_NO_BEST_FIT_CHARS, (wchar_t*)pucOriginalText, -1, NULL, 0, NULL, NULL);
		pucText = (unsigned char*)malloc(nBufSize);
		WideCharToMultiByte(CP_ACP, WC_NO_BEST_FIT_CHARS, (wchar_t*)pucOriginalText, -1, (LPSTR)pucText, nBufSize, NULL, NULL);
		return nBufSize;
	
	case UTF8_TO_SHIFTJIS:
		CharSet::ConvertText((unsigned char*)pucOriginalText, UTF8_TO_UCS2);
		if (nBufSize == -1) return 0;
		pucTempText = (unsigned char*)malloc(nBufSize);
		CharSet::CopyText((unsigned char*)pucTempText);
		CharSet::ConvertText((unsigned char*)pucTempText, UCS2_TO_SHIFTJIS);
		free(pucTempText);
		return nBufSize;
	
	}
	return 0;
}

//----------------------------------------
// Rs[֐
//----------------------------------------
void CharSet::CopyText(unsigned char* pucConvertedText)
{
	if (nBufSize == -1) return;
	memcpy(pucConvertedText, pucText, nBufSize);
	free(pucText);
	pucText = NULL;
}
