/*********************************************************************************
 
 MD5NX


 Copyright (c) 2005-2013, Isao MORI (Orios)
 All rights reserved.

 Redistribution and use in source and binary forms, with or without modification,
 are permitted provided that the following conditions are met:
   * Redistributions of source code must retain the above copyright notice, this
     list of conditions and the following disclaimer.
   * Redistributions in binary form must reproduce the above copyright notice,
     this list of conditions and the following disclaimer in the documentation
     and/or other materials provided with the distribution.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*********************************************************************************/

//==================================================
// CN[h
//==================================================
#include "MD5.h"

//----------------------------------------
//
//  F֐
//
//   [p[^]
//    (ȗ)
//
//   [߂l]
//    (unsigned long) Z
//----------------------------------------
unsigned long MD5::F(unsigned long X, unsigned long Y, unsigned long Z) const
{
	return (X & Y) | ((~X) & Z);
}

//----------------------------------------
//
//  G֐
//
//   [p[^]
//    (ȗ)
//
//   [߂l]
//    (unsigned long) Z
//----------------------------------------
unsigned long MD5::G(unsigned long X, unsigned long Y, unsigned long Z) const
{
	return (X & Z) | (Y & (~Z));
}

//----------------------------------------
//
//  H֐
//
//   [p[^]
//    (ȗ)
//
//   [߂l]
//    (unsigned long) Z
//----------------------------------------
unsigned long MD5::H(unsigned long X, unsigned long Y, unsigned long Z) const
{
	return X ^ Y ^ Z;
}

//----------------------------------------
//
//  I֐
//
//   [p[^]
//    (ȗ)
//
//   [߂l]
//    (unsigned long) Z
//----------------------------------------
unsigned long MD5::I(unsigned long X, unsigned long Y, unsigned long Z) const
{
	return Y ^ (X | (~Z));
}

//----------------------------------------
//
//  rbgVtg֐
//
//   [p[^]
//    (unsigned long) X: Vtg4oCgϐ
//    (short) n        : Vtg
//
//   [߂l]
//    (unsigned long) Z
//----------------------------------------
unsigned long MD5::ROTATE_LEFT(unsigned long X, short n) const
{
	return (X << n) | (X >> (32 - n));
}

//----------------------------------------
//
//  FF֐
//
//   [p[^]
//    (ȗ)
//
//   [߂l]
//    (Ȃ)
//----------------------------------------
void MD5::FF(unsigned long &a, unsigned long b, unsigned long c, unsigned long d, unsigned long Xk, short s, unsigned long Ti) const
{
	a = b + ROTATE_LEFT((a + F(b, c, d) + Xk + Ti), s);
}

//----------------------------------------
//
//  GG֐
//
//   [p[^]
//    (ȗ)
//
//   [߂l]
//    (Ȃ)
//----------------------------------------
void MD5::GG(unsigned long &a, unsigned long b, unsigned long c, unsigned long d, unsigned long Xk, short s, unsigned long Ti) const
{
	a = b + ROTATE_LEFT((a + G(b, c, d) + Xk + Ti), s);
}

//----------------------------------------
//
//  HH֐
//
//   [p[^]
//    (ȗ)
//
//   [߂l]
//    (Ȃ)
//----------------------------------------
void MD5::HH(unsigned long &a, unsigned long b, unsigned long c, unsigned long d, unsigned long Xk, short s, unsigned long Ti) const
{
	a = b + ROTATE_LEFT((a + H(b, c, d) + Xk + Ti), s);
}

//----------------------------------------
//
//  II֐
//
//   [p[^]
//    (ȗ)
//
//   [߂l]
//    (Ȃ)
//----------------------------------------
void MD5::II(unsigned long &a, unsigned long b, unsigned long c, unsigned long d, unsigned long Xk, short s, unsigned long Ti) const
{
	a = b + ROTATE_LEFT((a + I(b, c, d) + Xk + Ti), s);
}

//----------------------------------------
//
//  10i16iɕϊ֐
//
//   [p[^]
//    (unsigned long) ulDecNum: 10i
//    (char*) pcHexStr: ϊ16iۑobt@̃|C^
//
//   [߂l]
//    (Ȃ)
//----------------------------------------
void MD5::ConvDecToHex(unsigned long ulDecNum, char* pcHexStr) const
{
	// ϐ錾
	char cTemp;
	short i;

	// ϊ
	for (i = 0; i < 8; i++) {
		cTemp = (char)((ulDecNum >> (i * 4)) & 0x0F);
		if (cTemp < 10) {
			pcHexStr[7 - i] = cTemp + 48;
		}
		else {
			pcHexStr[7 - i] = cTemp + 55;
		}
	}
}

//----------------------------------------
//
//  MD5֐
//
//   [p[^]
//    (char*) pcData: nbV𐶐f[^
//    (DWORD) dwNumOfBytes: f[^̃TCY
//    (unsigned long*) pulResults: ʂۑobt@(16oCg)
//----------------------------------------
bool MD5::ProduceMD5(const char* pcData, DWORD dwNumOfBytes, unsigned long* pulResults) const
{
	// ϐ錾
	DWORD dwNeededSize;
	DWORD dwNumOfBlocks;
	DWORD dwLowNumOfBits;
	DWORD dwHighNumOfBits;
	unsigned char* pucTempData;
	unsigned long A, B, C, D;
	unsigned long X[16];
	unsigned long AA, BB, CC, DD;
	DWORD i;
	short j;

	// Kvf[^̌vZ
	if (dwNumOfBytes % 64 == 0) {
		dwNeededSize = dwNumOfBytes + 64;
	}
	else if ((dwNumOfBytes % 64 < 56)) {
		dwNeededSize = (DWORD)floor((double)dwNumOfBytes / 64) * 64 + 64;
	}
	else {
		dwNeededSize = (DWORD)floor((double)dwNumOfBytes / 64) * 64 + 128;
	}
	dwNumOfBlocks = dwNeededSize / 64;
	dwLowNumOfBits = dwNumOfBytes << 3;
	dwHighNumOfBits = dwNumOfBytes >> (8 * sizeof(DWORD) - 3);

	// MD5̌vZɎgpf[^̐
	pucTempData = (unsigned char*) malloc(dwNeededSize);
	if (pucTempData == 0) {
		return false;
	}
	FillMemory(pucTempData, dwNeededSize, 0);
	memcpy(pucTempData, pcData, dwNumOfBytes);
	pucTempData[dwNumOfBytes] = 0x80;
	memcpy(pucTempData + dwNeededSize - 8, &dwLowNumOfBits, 4);
	memcpy(pucTempData + dwNeededSize - 4, &dwHighNumOfBits, 4);

	// 
	A = 0x67452301;
	B = 0xEFCDAB89;
	C = 0x98BADCFE;
	D = 0x10325476;

	// MD5̌vZ
	for (i = 0; i < dwNumOfBlocks; i++) {

		// f[^̃Rs[(1)
		for (j = 0; j < 16; j++) {
			X[j] = (pucTempData[(i * 16 + j) * 4 + 3] << 24) | (pucTempData[(i * 16 + j) * 4 + 2] << 16) | (pucTempData[(i * 16 + j) * 4 + 1] << 8) | pucTempData[(i * 16 + j) * 4];
		}

		// f[^̃Rs[(2)
		AA = A;
		BB = B;
		CC = C;
		DD = D;

		// Round 1
		FF (A, B, C, D, X[ 0],  7, 0xd76aa478);
		FF (D, A, B, C, X[ 1], 12, 0xe8c7b756);
		FF (C, D, A, B, X[ 2], 17, 0x242070db);
		FF (B, C, D, A, X[ 3], 22, 0xc1bdceee);
		FF (A, B, C, D, X[ 4],  7, 0xf57c0faf);
		FF (D, A, B, C, X[ 5], 12, 0x4787c62a);
		FF (C, D, A, B, X[ 6], 17, 0xa8304613);
		FF (B, C, D, A, X[ 7], 22, 0xfd469501);
		FF (A, B, C, D, X[ 8],  7, 0x698098d8);
		FF (D, A, B, C, X[ 9], 12, 0x8b44f7af);
		FF (C, D, A, B, X[10], 17, 0xffff5bb1);
		FF (B, C, D, A, X[11], 22, 0x895cd7be);
		FF (A, B, C, D, X[12],  7, 0x6b901122);
		FF (D, A, B, C, X[13], 12, 0xfd987193);
		FF (C, D, A, B, X[14], 17, 0xa679438e);
		FF (B, C, D, A, X[15], 22, 0x49b40821);

		// Round 2
		GG (A, B, C, D, X[ 1],  5, 0xf61e2562);
		GG (D, A, B, C, X[ 6],  9, 0xc040b340);
		GG (C, D, A, B, X[11], 14, 0x265e5a51);
		GG (B, C, D, A, X[ 0], 20, 0xe9b6c7aa);
		GG (A, B, C, D, X[ 5],  5, 0xd62f105d);
		GG (D, A, B, C, X[10],  9, 0x02441453);
		GG (C, D, A, B, X[15], 14, 0xd8a1e681);
		GG (B, C, D, A, X[ 4], 20, 0xe7d3fbc8);
		GG (A, B, C, D, X[ 9],  5, 0x21e1cde6);
		GG (D, A, B, C, X[14],  9, 0xc33707d6);
		GG (C, D, A, B, X[ 3], 14, 0xf4d50d87);
		GG (B, C, D, A, X[ 8], 20, 0x455a14ed);
		GG (A, B, C, D, X[13],  5, 0xa9e3e905);
		GG (D, A, B, C, X[ 2],  9, 0xfcefa3f8);
		GG (C, D, A, B, X[ 7], 14, 0x676f02d9);
		GG (B, C, D, A, X[12], 20, 0x8d2a4c8a);

		// Round 3
		HH (A, B, C, D, X[ 5],  4, 0xfffa3942);
		HH (D, A, B, C, X[ 8], 11, 0x8771f681);
		HH (C, D, A, B, X[11], 16, 0x6d9d6122);
		HH (B, C, D, A, X[14], 23, 0xfde5380c);
		HH (A, B, C, D, X[ 1],  4, 0xa4beea44);
		HH (D, A, B, C, X[ 4], 11, 0x4bdecfa9);
		HH (C, D, A, B, X[ 7], 16, 0xf6bb4b60);
		HH (B, C, D, A, X[10], 23, 0xbebfbc70);
		HH (A, B, C, D, X[13],  4, 0x289b7ec6);
		HH (D, A, B, C, X[ 0], 11, 0xeaa127fa);
		HH (C, D, A, B, X[ 3], 16, 0xd4ef3085);
		HH (B, C, D, A, X[ 6], 23, 0x04881d05);
		HH (A, B, C, D, X[ 9],  4, 0xd9d4d039);
		HH (D, A, B, C, X[12], 11, 0xe6db99e5);
		HH (C, D, A, B, X[15], 16, 0x1fa27cf8);
		HH (B, C, D, A, X[ 2], 23, 0xc4ac5665);

		// Round 4
		II (A, B, C, D, X[ 0],  6, 0xf4292244);
		II (D, A, B, C, X[ 7], 10, 0x432aff97);
		II (C, D, A, B, X[14], 15, 0xab9423a7);
		II (B, C, D, A, X[ 5], 21, 0xfc93a039);
		II (A, B, C, D, X[12],  6, 0x655b59c3);
		II (D, A, B, C, X[ 3], 10, 0x8f0ccc92);
		II (C, D, A, B, X[10], 15, 0xffeff47d);
		II (B, C, D, A, X[ 1], 21, 0x85845dd1);
		II (A, B, C, D, X[ 8],  6, 0x6fa87e4f);
		II (D, A, B, C, X[15], 10, 0xfe2ce6e0);
		II (C, D, A, B, X[ 6], 15, 0xa3014314);
		II (B, C, D, A, X[13], 21, 0x4e0811a1);
		II (A, B, C, D, X[ 4],  6, 0xf7537e82);
		II (D, A, B, C, X[11], 10, 0xbd3af235);
		II (C, D, A, B, X[ 2], 15, 0x2ad7d2bb);
		II (B, C, D, A, X[ 9], 21, 0xeb86d391);

		// Final Process
		A += AA;
		B += BB;
		C += CC;
		D += DD;
	}

	// ʂ̏o
	pulResults[0] = ((A << 24) & 0xFF000000) | ((A << 8) & 0x00FF0000) | ((A >> 8) & 0x0000FF00) | ((A >> 16) & 0x000000FF);
	pulResults[1] = ((B << 24) & 0xFF000000) | ((B << 8) & 0x00FF0000) | ((B >> 8) & 0x0000FF00) | ((B >> 16) & 0x000000FF);
	pulResults[2] = ((C << 24) & 0xFF000000) | ((C << 8) & 0x00FF0000) | ((C >> 8) & 0x0000FF00) | ((C >> 16) & 0x000000FF);
	pulResults[3] = ((D << 24) & 0xFF000000) | ((D << 8) & 0x00FF0000) | ((D >> 8) & 0x0000FF00) | ((D >> 16) & 0x000000FF);

	// q[v̈̉
	free(pucTempData);

	// ߂l
	return true;
}

//----------------------------------------
//
//  MD5̃tH[}bg
//
//   [p[^]
//    (unsigned long*) pulResults: nbVf[^(16oCg)
//    (char*) pcMD5: ϊʂۑobt@
//
//   [߂l]
//    (Ȃ)
//----------------------------------------
void MD5::FormatMD5(const unsigned long* pulResults, char* pcMD5) const
{
	ConvDecToHex(pulResults[0], pcMD5);
	ConvDecToHex(pulResults[1], pcMD5 + 8);
	ConvDecToHex(pulResults[2], pcMD5 + 16);
	ConvDecToHex(pulResults[3], pcMD5 + 24);
}
