/*********************************************************************************
 
 RijndaelNX


 Copyright (c) 2005-2013, Isao MORI (Orios)
 All rights reserved.

 Redistribution and use in source and binary forms, with or without modification,
 are permitted provided that the following conditions are met:
   * Redistributions of source code must retain the above copyright notice, this
     list of conditions and the following disclaimer.
   * Redistributions in binary form must reproduce the above copyright notice,
     this list of conditions and the following disclaimer in the documentation
     and/or other materials provided with the distribution.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*********************************************************************************/

#include "Rijndael.h"

//----------------------------------------
// SBoxEtSBox̒`
//----------------------------------------
const unsigned char Rijndael::ucSBox[256] = {
	0x63, 0x7c, 0x77, 0x7b, 0xf2, 0x6b, 0x6f, 0xc5,
	0x30, 0x01, 0x67, 0x2b, 0xfe, 0xd7, 0xab, 0x76,
	0xca, 0x82, 0xc9, 0x7d, 0xfa, 0x59, 0x47, 0xf0,
	0xad, 0xd4, 0xa2, 0xaf, 0x9c, 0xa4, 0x72, 0xc0,
	0xb7, 0xfd, 0x93, 0x26, 0x36, 0x3f, 0xf7, 0xcc,
	0x34, 0xa5, 0xe5, 0xf1, 0x71, 0xd8, 0x31, 0x15,
	0x04, 0xc7, 0x23, 0xc3, 0x18, 0x96, 0x05, 0x9a,
	0x07, 0x12, 0x80, 0xe2, 0xeb, 0x27, 0xb2, 0x75,
	0x09, 0x83, 0x2c, 0x1a, 0x1b, 0x6e, 0x5a, 0xa0,
	0x52, 0x3b, 0xd6, 0xb3, 0x29, 0xe3, 0x2f, 0x84,
	0x53, 0xd1, 0x00, 0xed, 0x20, 0xfc, 0xb1, 0x5b,
	0x6a, 0xcb, 0xbe, 0x39, 0x4a, 0x4c, 0x58, 0xcf,
	0xd0, 0xef, 0xaa, 0xfb, 0x43, 0x4d, 0x33, 0x85,
	0x45, 0xf9, 0x02, 0x7f, 0x50, 0x3c, 0x9f, 0xa8,
	0x51, 0xa3, 0x40, 0x8f, 0x92, 0x9d, 0x38, 0xf5,
	0xbc, 0xb6, 0xda, 0x21, 0x10, 0xff, 0xf3, 0xd2,
	0xcd, 0x0c, 0x13, 0xec, 0x5f, 0x97, 0x44, 0x17,
	0xc4, 0xa7, 0x7e, 0x3d, 0x64, 0x5d, 0x19, 0x73,
	0x60, 0x81, 0x4f, 0xdc, 0x22, 0x2a, 0x90, 0x88,
	0x46, 0xee, 0xb8, 0x14, 0xde, 0x5e, 0x0b, 0xdb,
	0xe0, 0x32, 0x3a, 0x0a, 0x49, 0x06, 0x24, 0x5c,
	0xc2, 0xd3, 0xac, 0x62, 0x91, 0x95, 0xe4, 0x79,
	0xe7, 0xc8, 0x37, 0x6d, 0x8d, 0xd5, 0x4e, 0xa9,
	0x6c, 0x56, 0xf4, 0xea, 0x65, 0x7a, 0xae, 0x08,
	0xba, 0x78, 0x25, 0x2e, 0x1c, 0xa6, 0xb4, 0xc6,
	0xe8, 0xdd, 0x74, 0x1f, 0x4b, 0xbd, 0x8b, 0x8a,
	0x70, 0x3e, 0xb5, 0x66, 0x48, 0x03, 0xf6, 0x0e,
	0x61, 0x35, 0x57, 0xb9, 0x86, 0xc1, 0x1d, 0x9e,
	0xe1, 0xf8, 0x98, 0x11, 0x69, 0xd9, 0x8e, 0x94,
	0x9b, 0x1e, 0x87, 0xe9, 0xce, 0x55, 0x28, 0xdf,
	0x8c, 0xa1, 0x89, 0x0d, 0xbf, 0xe6, 0x42, 0x68,
	0x41, 0x99, 0x2d, 0x0f, 0xb0, 0x54, 0xbb, 0x16
};

const unsigned char Rijndael::ucInvSBox[256] = {
	0x52, 0x09, 0x6a, 0xd5, 0x30, 0x36, 0xa5, 0x38,
	0xbf, 0x40, 0xa3, 0x9e, 0x81, 0xf3, 0xd7, 0xfb,
	0x7c, 0xe3, 0x39, 0x82, 0x9b, 0x2f, 0xff, 0x87,
	0x34, 0x8e, 0x43, 0x44, 0xc4, 0xde, 0xe9, 0xcb,
	0x54, 0x7b, 0x94, 0x32, 0xa6, 0xc2, 0x23, 0x3d,
	0xee, 0x4c, 0x95, 0x0b, 0x42, 0xfa, 0xc3, 0x4e,
	0x08, 0x2e, 0xa1, 0x66, 0x28, 0xd9, 0x24, 0xb2,
	0x76, 0x5b, 0xa2, 0x49, 0x6d, 0x8b, 0xd1, 0x25,
	0x72, 0xf8, 0xf6, 0x64, 0x86, 0x68, 0x98, 0x16,
	0xd4, 0xa4, 0x5c, 0xcc, 0x5d, 0x65, 0xb6, 0x92,
	0x6c, 0x70, 0x48, 0x50, 0xfd, 0xed, 0xb9, 0xda,
	0x5e, 0x15, 0x46, 0x57, 0xa7, 0x8d, 0x9d, 0x84,
	0x90, 0xd8, 0xab, 0x00, 0x8c, 0xbc, 0xd3, 0x0a,
	0xf7, 0xe4, 0x58, 0x05, 0xb8, 0xb3, 0x45, 0x06,
	0xd0, 0x2c, 0x1e, 0x8f, 0xca, 0x3f, 0x0f, 0x02,
	0xc1, 0xaf, 0xbd, 0x03, 0x01, 0x13, 0x8a, 0x6b,
	0x3a, 0x91, 0x11, 0x41, 0x4f, 0x67, 0xdc, 0xea,
	0x97, 0xf2, 0xcf, 0xce, 0xf0, 0xb4, 0xe6, 0x73,
	0x96, 0xac, 0x74, 0x22, 0xe7, 0xad, 0x35, 0x85,
	0xe2, 0xf9, 0x37, 0xe8, 0x1c, 0x75, 0xdf, 0x6e,
	0x47, 0xf1, 0x1a, 0x71, 0x1d, 0x29, 0xc5, 0x89,
	0x6f, 0xb7, 0x62, 0x0e, 0xaa, 0x18, 0xbe, 0x1b,
	0xfc, 0x56, 0x3e, 0x4b, 0xc6, 0xd2, 0x79, 0x20,
	0x9a, 0xdb, 0xc0, 0xfe, 0x78, 0xcd, 0x5a, 0xf4,
	0x1f, 0xdd, 0xa8, 0x33, 0x88, 0x07, 0xc7, 0x31,
	0xb1, 0x12, 0x10, 0x59, 0x27, 0x80, 0xec, 0x5f,
	0x60, 0x51, 0x7f, 0xa9, 0x19, 0xb5, 0x4a, 0x0d,
	0x2d, 0xe5, 0x7a, 0x9f, 0x93, 0xc9, 0x9c, 0xef,
	0xa0, 0xe0, 0x3b, 0x4d, 0xae, 0x2a, 0xf5, 0xb0,
	0xc8, 0xeb, 0xbb, 0x3c, 0x83, 0x53, 0x99, 0x61,
	0x17, 0x2b, 0x04, 0x7e, 0xba, 0x77, 0xd6, 0x26,
	0xe1, 0x69, 0x14, 0x63, 0x55, 0x21, 0x0c, 0x7d
};

//----------------------------------------
//
//  GF8ł̏Z֐
//
//   [p[^]
//    (unsigned char) ucNumber: 搔
//    (short) sTimes         : 搔
//
//   [߂l]
//    (unsigned char) Z
//----------------------------------------
unsigned char Rijndael::GF8_Multi(unsigned char ucNumber, short sTimes) const
{
	// ϐ錾
	unsigned short sXTime;
	unsigned char ucReturn;

	// vZ
	sXTime = ucNumber;
	ucReturn = 0;
	do {
		if ((sTimes & 0x01) == 0x01) {
			ucReturn = ucReturn ^ sXTime;
		}
		if ((sXTime & 0x80) == 0x80) {
			sXTime = (sXTime << 1) ^ 0x1B;
		}
		else {
			sXTime = sXTime << 1;
		}
		sTimes = sTimes >> 1;
	} while (sTimes != 0);

	// ߂l
	return ucReturn;
}

//----------------------------------------
//
//  4oCgϐSBoxϊ
//
//   [p[^]
//    (unsigned long) ulNumber: ϊ4oCgϐ
//
//   [߂l]
//    (unsigned long) ϊς4oCgϐ
//----------------------------------------
unsigned long Rijndael::SubLong(unsigned long ulNumber) const
{
	return ucSBox[ulNumber & 0xFF] | (ucSBox[(ulNumber >> 8) & 0xFF] << 8) | (ucSBox[(ulNumber >> 16) & 0xFF] << 16) | (ucSBox[(ulNumber >> 24) & 0xFF] << 24);
}

//----------------------------------------
//
//  4oCgϐ1oCgVtg
//
//   [p[^]
//    (unsigned long) lNumber: Vtg4oCgϐ
//
//   [߂l]
//    (unsigned long) Vtgς4oCgϐ
//----------------------------------------
unsigned long Rijndael::RotBytes(unsigned long ulNumber) const
{
	return (ulNumber << 8) | (ulNumber >> 24);
}

//----------------------------------------
//
//  TBoxEtTBox̐
//
//   [p[^]
//    (Ȃ)
//
//   [߂l]
//    (Ȃ)
//----------------------------------------
void Rijndael::ProduceTBox()
{
	// ϐ錾
	short i;

	// TBoxEtTBox
	for (i = 0; i < 256; i++) {

		// TBox
		ulTBox[0][i] = GF8_Multi(ucSBox[i], 0x02) << 24 | (GF8_Multi(ucSBox[i], 0x01) << 16) | (GF8_Multi(ucSBox[i], 0x01) << 8) | (GF8_Multi(ucSBox[i], 0x03));
		ulTBox[1][i] = GF8_Multi(ucSBox[i], 0x03) << 24 | (GF8_Multi(ucSBox[i], 0x02) << 16) | (GF8_Multi(ucSBox[i], 0x01) << 8) | (GF8_Multi(ucSBox[i], 0x01));
		ulTBox[2][i] = GF8_Multi(ucSBox[i], 0x01) << 24 | (GF8_Multi(ucSBox[i], 0x03) << 16) | (GF8_Multi(ucSBox[i], 0x02) << 8) | (GF8_Multi(ucSBox[i], 0x01));
		ulTBox[3][i] = GF8_Multi(ucSBox[i], 0x01) << 24 | (GF8_Multi(ucSBox[i], 0x01) << 16) | (GF8_Multi(ucSBox[i], 0x03) << 8) | (GF8_Multi(ucSBox[i], 0x02));

		// tTBox
		ulInvTBox[0][i] = (GF8_Multi((unsigned char)i, 0x0E) << 24) | (GF8_Multi((unsigned char)i, 0x09) << 16) | (GF8_Multi((unsigned char)i, 0x0D) << 8) | (GF8_Multi((unsigned char)i, 0x0B));
		ulInvTBox[1][i] = (GF8_Multi((unsigned char)i, 0x0B) << 24) | (GF8_Multi((unsigned char)i, 0x0E) << 16) | (GF8_Multi((unsigned char)i, 0x09) << 8) | (GF8_Multi((unsigned char)i, 0x0D));
		ulInvTBox[2][i] = (GF8_Multi((unsigned char)i, 0x0D) << 24) | (GF8_Multi((unsigned char)i, 0x0B) << 16) | (GF8_Multi((unsigned char)i, 0x0E) << 8) | (GF8_Multi((unsigned char)i, 0x09));
		ulInvTBox[3][i] = (GF8_Multi((unsigned char)i, 0x09) << 24) | (GF8_Multi((unsigned char)i, 0x0D) << 16) | (GF8_Multi((unsigned char)i, 0x0B) << 8) | (GF8_Multi((unsigned char)i, 0x0E));
	}
}

//----------------------------------------
//
//  Í̃p[^ݒ
//
//   [p[^]
//    (short) sBlockLen: ubN(P:rbg)
//    (short) sKeyLen  : (P:rbg)
//
//   [߂l]
//    (bool) true: , false: s
//----------------------------------------
bool Rijndael::SetParameters(short sBlockLen, short sKeyLen)
{
	// ϐ錾
	char cShift[4];
	short i, j;

	// p[^̃`FbN
	if (((sBlockLen % 32) != 0) || ((sKeyLen % 32) != 0) || (sBlockLen < 128) || (sBlockLen > 256)) {
		return false;
	}

	// {p[^̐ݒ
	Nb = sBlockLen / 32;
	Nk = sKeyLen / 32;
	sBytesOfBlock = Nb * 4;
	sBytesOfKey = Nk * 4;
	Nb2 = Nb * 2;
	Nb3 = Nb * 3;

	// Eȟ
	if (Nb > Nk) {
		Nr = Nb + 6;
	}
	else {
		Nr = Nk + 6;
	}

	// Vtg̐ݒ
	cShift[0] = 0;
	cShift[1] = 1;
	if (Nb == 8) {
		cShift[2] = 3;
		cShift[3] = 4;
	}
	else if ((Nb == 7)) {
		cShift[2] = 2;
		cShift[3] = 4;
	}
	else {
		cShift[2] = 2;
		cShift[3] = 3;
	}

	// ꎞƗp̈̊m
	if (apucTempBlock[0] == NULL) {
		apucTempBlock[0] = (unsigned char*) malloc(sBytesOfBlock);
	}
	else {
		apucTempBlock[0] = (unsigned char*) realloc(apucTempBlock[0], sBytesOfBlock);
	}
	if (apucTempBlock[1] == NULL) {
		apucTempBlock[1] = (unsigned char*) malloc(sBytesOfBlock);
	}
	else {
		apucTempBlock[1] = (unsigned char*) realloc(apucTempBlock[1], sBytesOfBlock);
	}
	if ((apucTempBlock[0] == NULL) || (apucTempBlock[1] == NULL)) {
		return false;
	}
	
	// gp̈̊m
	if (pucExpandedKey == NULL) {
		pucExpandedKey = (unsigned char*) malloc(sBytesOfBlock * (Nr + 1));
	}
	else {
		pucExpandedKey = (unsigned char*) realloc(pucExpandedKey, sBytesOfBlock * (Nr + 1));
	}
	if (pucExpandedKey == NULL) {
		return false;
	}
	
	// Vtgʒu̕ϊe[u̐
	if (pcShift == NULL) {
		pcShift = (char*) malloc(sBytesOfBlock);
	}
	else {
		pcShift = (char*) realloc(pcShift, sBytesOfBlock);
	}
	if (pcInvShift == NULL) {
		pcInvShift = (char*) malloc(sBytesOfBlock);
	}
	else {
		pcInvShift = (char*) realloc(pcInvShift, sBytesOfBlock);
	}
	if ((pcShift == NULL) || (pcInvShift == NULL)) {
		return false;
	}
	for (j = 0; j < 4; j++) {
		for (i = 0; i < Nb; i++) {
			pcShift[Nb * j + i] = Nb * j + (i + Nb - cShift[j]) % Nb;
			pcInvShift[Nb * j + i] = Nb * j + (i + cShift[j]) % Nb;
		}
	}

	// ߂l
	return true;
}

//----------------------------------------
//
//  g֐
//
//   [p[^]
//    (unsigned char*) pucKey        : ̌
//    (unsigned char*) pucExpandedKey: gꂽi[obt@̃|C^
//
//   [߂l]
//    (bool) true: , false: s
//----------------------------------------
bool Rijndael::KeyExpansion(const char* pcKey, unsigned char* pucExpandedKey) const
{
	// ϐ̐錾
	unsigned long* plRcon;
	unsigned long ulTemp1, ulTemp2;
	short i;

	// lRcon̒l̐
	plRcon = (unsigned long*) malloc((Nb * (Nr + 1) / Nk + 1) * sizeof(unsigned long));
	if (plRcon == NULL) {
		return false;
	}
	plRcon[1] = 0x01000000;
	for (i = 2; i < (Nb * (Nr + 1) / Nk + 1); i++) {
		if ((plRcon[i - 1] & 0x80000000) == 0x80000000) {
			plRcon[i] = (unsigned long)((plRcon[i - 1] << 1) ^ 0x1B000000);
		}
		else {
			plRcon[i] = plRcon[i - 1] << 1;
		}
	}

	// g
	memcpy(pucExpandedKey, pcKey, Nk * 4);
	for (i = Nk; i < Nb * (Nr + 1); i++) {
		ulTemp1 = (pucExpandedKey[4 * i - 4] << 24) | (pucExpandedKey[4 * i - 3] << 16) | (pucExpandedKey[4 * i - 2] << 8) | (pucExpandedKey[4 * i - 1]);
		if (i % Nk == 0) {
			ulTemp1 = SubLong(RotBytes(ulTemp1)) ^ plRcon[i / Nk];
		}
		else if (((Nk > 6) && (i % Nk == 4))) {
			ulTemp1 = SubLong(ulTemp1);
		}
		ulTemp2 = (pucExpandedKey[(i - Nk) * 4] << 24) | (pucExpandedKey[(i - Nk) * 4 + 1] << 16) | (pucExpandedKey[(i - Nk) * 4 + 2] << 8) | (pucExpandedKey[(i - Nk) * 4 + 3]);
		ulTemp2 ^= ulTemp1;
		for (short j = 0; j < 4; j++) {
			pucExpandedKey[i * 4 + j] = (unsigned char)(ulTemp2 >> (24 - j * 8));
		}
	}

	// q[v̈̉
	free(plRcon);

	// ߂l
	return true;
}

//----------------------------------------
//
//  Eh֐
//
//   [p[^]
//    (unsigned char*) pucEncBlockData: Íf[^i[obt@̃|C^
//    (unsinged char*) pucTempBlock: Ɨp̈̃|C^ (ubNKv)
//    (unsigned char*) pucRoundKey : ÍɎgp錮
//    (bool) bFinal: ŏIEhtO (true: ŏIEh, false: ʏ탉Eh)
//
//   [߂l]
//    (Ȃ)
//----------------------------------------
void Rijndael::Round(unsigned char* pucEncBlockData, unsigned char* pucTempBlock, const unsigned char* pucRoundKey, bool bFinal) const
{	
	// ŏIEh
	if (bFinal) {
		for (k = 0; k < Nb; k++) {
			pucTempBlock[k] = ucSBox[pucEncBlockData[pcInvShift[k]]] ^ pucRoundKey[k * 4];
			pucTempBlock[Nb + k] = ucSBox[pucEncBlockData[pcInvShift[Nb + k]]] ^ pucRoundKey[k * 4 + 1];
			pucTempBlock[Nb2 + k] = ucSBox[pucEncBlockData[pcInvShift[Nb2 + k]]] ^ pucRoundKey[k * 4 + 2];
			pucTempBlock[Nb3 + k] = ucSBox[pucEncBlockData[pcInvShift[Nb3 + k]]] ^ pucRoundKey[k * 4 + 3];
		}

	}
	// ʏ탉Eh
	else {
		for (k = 0; k < Nb; k++) {
			ulTemp = ulTBox[0][pucEncBlockData[pcInvShift[k]]] ^ ulTBox[1][pucEncBlockData[pcInvShift[Nb + k]]] ^ ulTBox[2][pucEncBlockData[pcInvShift[Nb2 + k]]] ^ ulTBox[3][pucEncBlockData[pcInvShift[Nb3 + k]]];
			pucTempBlock[k] = (unsigned char)((ulTemp >> 24) ^ pucRoundKey[k * 4]);
			pucTempBlock[Nb + k] = (unsigned char)((ulTemp >> 16) ^ pucRoundKey[k * 4 + 1]);
			pucTempBlock[Nb2 + k] = (unsigned char)((ulTemp >> 8) ^ pucRoundKey[k * 4 + 2]);
			pucTempBlock[Nb3 + k] = (unsigned char)(ulTemp ^ pucRoundKey[k * 4 + 3]);
		}
	}
}

//----------------------------------------
//
//  tEh֐
//
//   [p[^]
//    (unsigned char*) pucDecBlockData  : f[^i[obt@̃|C^
//    (unsigned char*) pucTempBlock: Ɨp̈̃|C^ (ubNKv)
//    (unsigned char*) pucRoundKey: Ɏgp錮
//    (bool) bFinal: ŏIEhtO (true: ŏIEh, false: ʏ탉Eh)
//
//   [߂l]
//    (Ȃ)
//----------------------------------------
void Rijndael::InvRound(unsigned char* pucDecBlockData, unsigned char* pucTempBlock, const unsigned char* pucRoundKey, bool bFinal) const
{
	// ŏIEh
	if (bFinal) {
		for (k = 0; k < Nb; k++) {
			pucTempBlock[pcInvShift[k]] = ucInvSBox[(pucDecBlockData[k] ^ pucRoundKey[k * 4])];
			pucTempBlock[pcInvShift[Nb + k]] = ucInvSBox[(pucDecBlockData[Nb + k] ^ pucRoundKey[k * 4 + 1])];
			pucTempBlock[pcInvShift[Nb2 + k]] = ucInvSBox[(pucDecBlockData[Nb2 + k] ^ pucRoundKey[k * 4 + 2])];
			pucTempBlock[pcInvShift[Nb3 + k]] = ucInvSBox[(pucDecBlockData[Nb3 + k] ^ pucRoundKey[k * 4 + 3])];
		}
	}

	// ʏ탉Eh
	else {
		for (k = 0; k < Nb; k++) {
			ulTemp = ulInvTBox[0][(pucDecBlockData[k] ^ pucRoundKey[k * 4])]
			       ^ ulInvTBox[1][(pucDecBlockData[Nb + k] ^ pucRoundKey[k * 4 + 1])]
				   ^ ulInvTBox[2][(pucDecBlockData[Nb2 + k] ^ pucRoundKey[k * 4 + 2])]
				   ^ ulInvTBox[3][(pucDecBlockData[Nb3 + k] ^ pucRoundKey[k * 4 + 3])];
			pucTempBlock[pcInvShift[k]] = ucInvSBox[(ulTemp >> 24) & 0xFF];
			pucTempBlock[pcInvShift[Nb + k]] = ucInvSBox[(ulTemp >> 16) & 0xFF];
			pucTempBlock[pcInvShift[Nb2 + k]] = ucInvSBox[(ulTemp >> 8) & 0xFF];
			pucTempBlock[pcInvShift[Nb3 + k]] = ucInvSBox[(ulTemp) & 0xFF];
		}
	}
}

//----------------------------------------
//
//  RXgN^
//
//   [p[^]
//    (Ȃ)
//
//   [߂l]
//    (Ȃ)
//----------------------------------------
Rijndael::Rijndael()
{
	ProduceTBox();
	pcShift = NULL;
	pcInvShift = NULL;
	apucTempBlock[0] = NULL;
	apucTempBlock[1] = NULL;
	pucExpandedKey = NULL;
	hWndToSend = 0;
	wMsgToSend = 0;
	SetParameters(128, 256);
}

//----------------------------------------
//
//  fXgN^
//
//   [p[^]
//    (Ȃ)
//
//   [߂l]
//    (Ȃ)
//----------------------------------------
Rijndael::~Rijndael()
{
	if (pcShift != NULL) {
		free(pcShift);
	}
	if (pcInvShift != NULL) {
		free(pcInvShift);
	}
	if (apucTempBlock[0] != NULL) {
		free(apucTempBlock[0]);
	}
	if (apucTempBlock[1] != NULL) {
		free(apucTempBlock[1]);
	}
	if (pucExpandedKey != NULL) {
		free(pucExpandedKey);
	}
}

//----------------------------------------
//
//  obt@TCY擾֐
//
//   [p[^]
//    (unsigned __int64) un64NumOfBytes: Íf[^̃TCY
//
//   [߂l]
//    (unsigned __int64) Kvȃobt@TCY
//----------------------------------------
unsigned __int64 Rijndael::GetNeededBufSize(unsigned __int64 un64NumOfBytes) const
{
	if (un64NumOfBytes % sBytesOfBlock == 0) {
		return un64NumOfBytes;
	}
	else {
		return (unsigned __int64)floor((double)(un64NumOfBytes / sBytesOfBlock)) * sBytesOfBlock + sBytesOfBlock;
	}
}

//----------------------------------------
//
//  Í֐
//
//   [p[^]
//    (unsigned char*) pucData: Íf[^
//    (DWORD) dwNumOfBytes: f[^̃TCY
//    (char*) pcEncKey: ÍɎgp錮
//    (unsigned char*) pucEncData: Íꂽf[^i[obt@̃|C^
//    (unsigned char*) pucVector: CBCpxNg
//
//   [߂l]
//    (bool) true: , false: s
//----------------------------------------
bool Rijndael::Encrypt(const unsigned char* pucData, DWORD dwNumOfBytes, const char* pcEncKey, unsigned char* pucEncData, unsigned char* pucVector)
{

	// ϐ錾
	DWORD dwNumOfBlocks;
	DWORD dwCurPos;
	char cInBlockNum;
	char cOutBlockNum;
	DWORD i;
	short j;

	// g
	if (!KeyExpansion(pcEncKey, pucExpandedKey)) {
		return false;
	}

	// ubŇ
	if (dwNumOfBytes % sBytesOfBlock == 0) {
		dwNumOfBlocks = dwNumOfBytes / sBytesOfBlock;
	}
	else {
		dwNumOfBlocks = (DWORD)floor((double)(dwNumOfBytes / sBytesOfBlock)) + 1;
	}

	// Eh̎s
	for (i = 0; i < dwNumOfBlocks; i++) {

		// ItZbgʒǔvZ
		dwCurPos = i * sBytesOfBlock;

		// IWif[^̃Rs[
		for (j = 0; j < Nb; j++) {
			apucTempBlock[0][j] = pucData[dwCurPos + j * 4] ^ pucVector[j * 4] ^ pucExpandedKey[j * 4];
			apucTempBlock[0][Nb + j] = pucData[dwCurPos + j * 4 + 1] ^ pucVector[j * 4 + 1] ^ pucExpandedKey[j * 4 + 1];
			apucTempBlock[0][Nb2 + j] = pucData[dwCurPos + j * 4 + 2] ^ pucVector[j * 4 + 2] ^ pucExpandedKey[j * 4 + 2];
			apucTempBlock[0][Nb3 + j] = pucData[dwCurPos + j * 4 + 3] ^ pucVector[j * 4 + 3] ^ pucExpandedKey[j * 4 + 3];
		}

		// Eh̎s
		cInBlockNum = 0;
		cOutBlockNum = 1;
		for (j = 1; j < Nr; j++) {
			Round(apucTempBlock[cInBlockNum], apucTempBlock[cOutBlockNum], pucExpandedKey + j * sBytesOfBlock, false);
			cInBlockNum ^= 0x01;
			cOutBlockNum ^= 0x01;
		}
		Round(apucTempBlock[cInBlockNum], apucTempBlock[cOutBlockNum], pucExpandedKey + Nr * sBytesOfBlock, true);

		// Íσf[^̃Rs[
		for (j = 0; j < Nb; j++) {
			pucEncData[dwCurPos + j * 4] = pucVector[j * 4] = apucTempBlock[cOutBlockNum][j];
			pucEncData[dwCurPos + j * 4 + 1] = pucVector[j * 4 + 1] = apucTempBlock[cOutBlockNum][Nb + j];
			pucEncData[dwCurPos + j * 4 + 2] = pucVector[j * 4 + 2] = apucTempBlock[cOutBlockNum][Nb2 + j];
			pucEncData[dwCurPos + j * 4 + 3] = pucVector[j * 4 + 3] = apucTempBlock[cOutBlockNum][Nb3 + j];
		}

		if (((i & 0x00007FF) == 0) && (hWndToSend != 0)) {
			PostMessage(hWndToSend, wMsgToSend, 0, dwCurPos);
		}
	}

	// ߂l
	return true;
}

//----------------------------------------
//
//  ֐
//
//   [p[^]
//    (unsigned char*) pucData: f[^
//    (DWORD) dwNumOfBytes: f[^̃TCY
//    (char*) pcDecKey : Ɏgp錮
//    (unsigned char*) pucDecData: ꂽf[^i[obt@̃|C^
//    (unsigned char*) pucInitVector: CBCpxNg
//
//   [߂l]
//    (bool) true: , false: s
//----------------------------------------
bool Rijndael::Decrypt(const unsigned char* pucData, DWORD dwNumOfBytes, const char* pcDecKey, unsigned char* pucDecData, unsigned char* pucInitVector)
{
	// ϐ錾
	unsigned long dwNumOfBlocks;
	DWORD dwCurPos;
	char cDecBlockNum;
	char cTempBlockNum;
	DWORD i;
	short j;

	// Í̐ݒ
	if (!KeyExpansion(pcDecKey, pucExpandedKey)) {
		return false;
	}

	// ubŇ
	dwNumOfBlocks = (DWORD)floor((double)(dwNumOfBytes / sBytesOfBlock));

	// tϊEh̎s
	i = dwNumOfBlocks - 1;
	while (true) {
		dwCurPos = i * sBytesOfBlock;

		// IWif[^̃Rs[
		for (j = 0; j < Nb; j++) {
			apucTempBlock[0][j] = pucData[dwCurPos + j * 4];
			apucTempBlock[0][Nb + j] = pucData[dwCurPos + j * 4 + 1];
			apucTempBlock[0][Nb2 + j] = pucData[dwCurPos + j * 4 + 2];
			apucTempBlock[0][Nb3 + j] = pucData[dwCurPos + j * 4 + 3];
		}

		// Eh̎s
		cDecBlockNum = 0;
		cTempBlockNum = 1;
		InvRound(apucTempBlock[cDecBlockNum], apucTempBlock[cTempBlockNum], pucExpandedKey + Nr * sBytesOfBlock, true);
		for (j = Nr - 1; j > 0; j--) {
			cDecBlockNum ^= 0x01;
			cTempBlockNum ^= 0x01;
			InvRound(apucTempBlock[cDecBlockNum], apucTempBlock[cTempBlockNum], pucExpandedKey + j * sBytesOfBlock, false);
		}
		
			// f[^̃Rs[
		if (i == 0) {
			// IWif[^̃Rs[
			for (j = 0; j < Nb; j++) {
				pucDecData[dwCurPos + j * 4] = apucTempBlock[cTempBlockNum][j] ^ pucInitVector[j * 4] ^ pucExpandedKey[j * 4];
				pucDecData[dwCurPos + j * 4 + 1] = apucTempBlock[cTempBlockNum][Nb + j] ^ pucInitVector[j * 4 + 1] ^ pucExpandedKey[j * 4 + 1];
				pucDecData[dwCurPos + j * 4 + 2] = apucTempBlock[cTempBlockNum][Nb2 + j] ^ pucInitVector[j * 4 + 2] ^ pucExpandedKey[j * 4 + 2];
				pucDecData[dwCurPos + j * 4 + 3] = apucTempBlock[cTempBlockNum][Nb3 + j] ^ pucInitVector[j * 4 + 3] ^ pucExpandedKey[j * 4 + 3];
			}
			break;
		}
		else {
			for (j = 0; j < Nb; j++) {
				pucDecData[dwCurPos + j * 4] = apucTempBlock[cTempBlockNum][j] ^ pucData[dwCurPos + j * 4 - sBytesOfBlock] ^ pucExpandedKey[j * 4];
				pucDecData[dwCurPos + j * 4 + 1] = apucTempBlock[cTempBlockNum][Nb + j] ^ pucData[dwCurPos + j * 4 + 1 - sBytesOfBlock] ^ pucExpandedKey[j * 4 + 1];
				pucDecData[dwCurPos + j * 4 + 2] = apucTempBlock[cTempBlockNum][Nb2 + j] ^ pucData[dwCurPos + j * 4 + 2 - sBytesOfBlock] ^ pucExpandedKey[j * 4 + 2];
				pucDecData[dwCurPos + j * 4 + 3] = apucTempBlock[cTempBlockNum][Nb3 + j] ^ pucData[dwCurPos + j * 4 + 3 - sBytesOfBlock] ^ pucExpandedKey[j * 4 + 3];
			}
		}

		// EBhEbZ[W̑M
		if (((i & 0x000003FF) == 0) && (hWndToSend != 0)) {
			PostMessage(hWndToSend, wMsgToSend, 1, dwCurPos);
		}
		i--;
	}

	// ߂l
	return true;
}

//----------------------------------------
//
//  bZ[WMEBhEo^֐
//
//   [p[^]
//    (HWND) hWnd: MEBhE
//    (UINT) wMsg: bZ[W
//
//   [߂l]
//    (Ȃ)
//----------------------------------------
void Rijndael::RegWndToSendMsg(HWND hWnd, UINT wMsg)
{
	hWndToSend = hWnd;
	wMsgToSend = wMsg;
}
