/*********************************************************************************
 
 SHANXQ


 Copyright (c) 2005-2013, Isao MORI (Orios)
 All rights reserved.

 Redistribution and use in source and binary forms, with or without modification,
 are permitted provided that the following conditions are met:
   * Redistributions of source code must retain the above copyright notice, this
     list of conditions and the following disclaimer.
   * Redistributions in binary form must reproduce the above copyright notice,
     this list of conditions and the following disclaimer in the documentation
     and/or other materials provided with the distribution.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*********************************************************************************/

#include "SHA.h"

//----------------------------------------
//
//  rbgVtg()֐
//
//   [p[^]
//    (unsigned long) X: Vtg4oCgϐ
//    (short) n        : Vtg
//
//   [߂l]
//    (unsigned long) Vtg
//----------------------------------------
unsigned long SHABase::ROTL(unsigned long X, short n) const
{
	return (X << n) | (X >> (32 - n));
}

//----------------------------------------
//
//  rbgVtg(E)֐
//
//   [p[^]
//    (unsigned long) X: Vtg4oCgϐ
//    (short) n        : Vtg
//
//   [߂l]
//    (unsigned long) Vtg
//----------------------------------------
unsigned long SHABase::ROTR(unsigned long X, short n) const
{
	return (X >> n) | (X << (32 - n));
}

//----------------------------------------
//
//  Ch֐
//
//   [p[^]
//    (ȗ)
//
//   [߂l]
//    (unsigned long) Z
//----------------------------------------
unsigned long SHABase::Ch(unsigned long B, unsigned long C, unsigned long D) const
{
	return (B & C) ^ ((~B) & D);
}

//----------------------------------------
//
//  Parity֐
//
//   [p[^]
//    (ȗ)
//
//   [߂l]
//    (unsigned long) Z
//----------------------------------------
unsigned long SHABase::Parity(unsigned long B, unsigned long C, unsigned long D) const
{
	return B ^ C ^ D;
}

//----------------------------------------
//
//  Maj֐
//
//   [p[^]
//    (ȗ)
//
//   [߂l]
//    (unsigned long) Z
//----------------------------------------
unsigned long SHABase::Maj(unsigned long B, unsigned long C, unsigned long D) const
{
	return (B & C) ^ (B & D) ^ (C & D);
}

//----------------------------------------
//
//  0֐
//
//   [p[^]
//    (ȗ)
//
//   [߂l]
//    (unsigned long) Z
//----------------------------------------
unsigned long SHABase::LSigma0(unsigned long X) const
{
	 return ROTR(X, 2) ^ ROTR(X, 13) ^ ROTR(X, 22);
}

//----------------------------------------
//
//  1֐
//
//   [p[^]
//    (ȗ)
//
//   [߂l]
//    (unsigned long) Z
//----------------------------------------
unsigned long SHABase::LSigma1(unsigned long X) const
{
	 return ROTR(X, 6) ^ ROTR(X, 11) ^ ROTR(X, 25);
}

//----------------------------------------
//
//  0֐
//
//   [p[^]
//    (ȗ)
//
//   [߂l]
//    (unsigned long) Z
//----------------------------------------
unsigned long SHABase::SSigma0(unsigned long X) const
{
	 return ROTR(X, 7) ^ ROTR(X, 18) ^ (X >> 3);
}

//----------------------------------------
//
//  1֐
//
//   [p[^]
//    (ȗ)
//
//   [߂l]
//    (unsigned long) Z
//----------------------------------------
unsigned long SHABase::SSigma1(unsigned long X) const
{
	 return ROTR(X, 17) ^ ROTR(X, 19) ^ (X >> 10);
}

//----------------------------------------
//
//  10i16iɕϊ֐
//
//   [p[^]
//    (unsigned long) lDecimalNum: 10i
//    (short) pucHexStr             : ϊ16iۑobt@̃|C^
//
//   [߂l]
//    (Ȃ)
//----------------------------------------
void SHABase::ConvDecToHex(unsigned long ulDecNum, char *pcHexStr) const
{
	// ϐ錾
	char cTemp;
	short i;

	// ϊ
	for (i = 0; i < 8; i++) {
		cTemp = (char)((ulDecNum >> (i * 4)) & 0x0F);
		if (cTemp < 10) {
			pcHexStr[7 - i] = cTemp + 48;
		}
		else {
			pcHexStr[7 - i] = cTemp + 55;
		}
	}
}


//----------------------------------------
//
//  SHA1֐
//
//   [p[^]
//    (unsigned char*) pucData   : nbV𐶐f[^
//    (DWORD) dwNumOfBytes    : f[^̃TCY
//    (unsigned long*) pulResults: ʂۑobt@(20oCg)
//
//   [߂l]
//    (bool) true: , false: s
//----------------------------------------
bool SHA1::ProduceSHA1(const unsigned char* pucData, DWORD dwNumOfBytes, unsigned long* pulResults) const
{
	// ϐ錾
	DWORD dwNeededSize;
	DWORD dwNumOfBlocks;
	DWORD dwLowNumOfBits;
	DWORD dwHighNumOfBits;
	unsigned char* pucTempData;
	unsigned long H0, H1, H2, H3, H4;
	unsigned long W[80];
	unsigned long A, B, C, D, E;
	unsigned long TEMP;
	DWORD i;
	short j;

	// Kvf[^̌vZ
	if (dwNumOfBytes % 64 == 0) {
		dwNeededSize = dwNumOfBytes + 64;
	}
	else if ((dwNumOfBytes % 64 < 56)) {
		dwNeededSize = (DWORD)floor((double)dwNumOfBytes / 64) * 64 + 64;
	}
	else {
		dwNeededSize = (DWORD)floor((double)dwNumOfBytes / 64) * 64 + 128;
	}
	dwNumOfBlocks = dwNeededSize / 64;
	dwLowNumOfBits = dwNumOfBytes << 3;
	dwHighNumOfBits = dwNumOfBytes >> (8 * sizeof(DWORD) - 3);

	// SHA1̌vZɎgpf[^̐
	pucTempData = (unsigned char*) malloc(dwNeededSize);
	if (pucTempData == 0) {
		return false;
	}
	FillMemory(pucTempData, dwNeededSize, 0);
	memcpy(pucTempData, pucData, dwNumOfBytes);
	pucTempData[dwNumOfBytes] = 0x80;
	for (i = 0; i < 4; i++) {
		pucTempData[dwNeededSize - 8 + i] = (unsigned char)((dwHighNumOfBits >> (24 - i * 8)) & 0xFF);
	}
	for (i = 0; i < 4; i++) {
		pucTempData[dwNeededSize - 4 + i] = (unsigned char)((dwLowNumOfBits >> (24 - i * 8)) & 0xFF);
	}

	// 
	H0 = 0x67452301;
	H1 = 0xEFCDAB89;
	H2 = 0x98BADCFE;
	H3 = 0x10325476;
	H4 = 0xC3D2E1F0;

	// SHA1̌vZ
	for (i = 0; i < dwNumOfBlocks; i++) {
		
		// Process.A.
		for (j = 0; j < 16; j++) {
			W[j] = (pucTempData[i * 64 + j * 4] << 24) | (pucTempData[i * 64 + j * 4 + 1] << 16) | (pucTempData[i * 64 + j * 4 + 2] << 8) | pucTempData[i * 64 + j * 4 + 3];
		}

		// Process.B.
		for (j = 16; j < 80; j++) {
			W[j] = ROTL(W[j - 3] ^ W[j - 8] ^ W[j - 14] ^ W[j - 16], 1);
		}

		// Process.C.
		A = H0;
		B = H1;
		C = H2;
		D = H3;
		E = H4;

		// Process.D.
		for (j = 0; j < 20; j++) {
			TEMP = ROTL(A, 5) + Ch(B, C, D) + E + W[j] + 0x5A827999;
			E = D;
			D = C;
			C = ROTL(B, 30);
			B = A;
			A = TEMP;
		}
		for (j = 20; j < 40; j++) {
			TEMP = ROTL(A, 5) + Parity(B, C, D) + E + W[j] + 0x6ED9EBA1;
			E = D;
			D = C;
			C = ROTL(B, 30);
			B = A;
			A = TEMP;
		}
		for (j = 40; j < 60; j++) {
			TEMP = ROTL(A, 5) + Maj(B, C, D) + E + W[j] + 0x8F1BBCDC;
			E = D;
			D = C;
			C = ROTL(B, 30);
			B = A;
			A = TEMP;
		}
		for (j = 60; j < 80; j++) {
			TEMP = ROTL(A, 5) + Parity(B, C, D) + E + W[j] + 0xCA62C1D6;
			E = D;
			D = C;
			C = ROTL(B, 30);
			B = A;
			A = TEMP;
		}
		
		// Process.E.
		H0 += A;
		H1 += B;
		H2 += C;
		H3 += D;
		H4 += E;
	}

	// ʂ̏o
	pulResults[0] = H0;
	pulResults[1] = H1;
	pulResults[2] = H2;
	pulResults[3] = H3;
	pulResults[4] = H4;

	// q[v̈̉
	free(pucTempData);

	// ߂l
	return true;
}

//----------------------------------------
//
//  SHA1̃tH[}bg
//
//   [p[^]
//    (unsigned long*) pulResults: nbVf[^(20oCg)
//    (unsigned char*) pcSHA1   : ϊʂۑobt@
//
//   [߂l]
//    (Ȃ)
//----------------------------------------
void SHA1::FormatSHA1(const unsigned long* pulResults, char* pcSHA1) const
{
	ConvDecToHex(pulResults[0], pcSHA1);
	ConvDecToHex(pulResults[1], pcSHA1 + 8);
	ConvDecToHex(pulResults[2], pcSHA1 + 16);
	ConvDecToHex(pulResults[3], pcSHA1 + 24);
	ConvDecToHex(pulResults[4], pcSHA1 + 32);
}



//----------------------------------------
//  萔K̐錾
//----------------------------------------
const unsigned long SHA256::K[64] = {
	0x428a2f98, 0x71374491, 0xb5c0fbcf, 0xe9b5dba5, 0x3956c25b, 0x59f111f1, 0x923f82a4, 0xab1c5ed5,
	0xd807aa98, 0x12835b01, 0x243185be, 0x550c7dc3, 0x72be5d74, 0x80deb1fe, 0x9bdc06a7, 0xc19bf174,
	0xe49b69c1, 0xefbe4786, 0x0fc19dc6, 0x240ca1cc, 0x2de92c6f, 0x4a7484aa, 0x5cb0a9dc, 0x76f988da,
	0x983e5152, 0xa831c66d, 0xb00327c8, 0xbf597fc7, 0xc6e00bf3, 0xd5a79147, 0x06ca6351, 0x14292967,
	0x27b70a85, 0x2e1b2138, 0x4d2c6dfc, 0x53380d13, 0x650a7354, 0x766a0abb, 0x81c2c92e, 0x92722c85,
	0xa2bfe8a1, 0xa81a664b, 0xc24b8b70, 0xc76c51a3, 0xd192e819, 0xd6990624, 0xf40e3585, 0x106aa070,
	0x19a4c116, 0x1e376c08, 0x2748774c, 0x34b0bcb5, 0x391c0cb3, 0x4ed8aa4a, 0x5b9cca4f, 0x682e6ff3,
	0x748f82ee, 0x78a5636f, 0x84c87814, 0x8cc70208, 0x90befffa, 0xa4506ceb, 0xbef9a3f7, 0xc67178f2
};

//----------------------------------------
//
//  SHA256֐
//
//   [p[^]
//    (unsigned char*) pucData   : nbV𐶐f[^
//    (DWORD) dwNumOfBytes    : f[^̃TCY
//    (unsigned long*) pulResults: ʂۑobt@(32oCg)
//
//   [߂l]
//    (bool) true: , false: s
//----------------------------------------
bool SHA256::ProduceSHA256(const unsigned char *pucData, DWORD dwNumOfBytes, unsigned long *pulResults) const
{
	// ϐ錾
	DWORD dwNeededSize;
	DWORD dwNumOfBlocks;
	DWORD dwLowNumOfBits;
	DWORD dwHighNumOfBits;
	unsigned char* pucTempData;
	unsigned long H0, H1, H2, H3, H4, H5, H6, H7;
	unsigned long W[64];
	unsigned long a, b, c, d, e, f, g, h;
	unsigned long T1, T2;
	DWORD i;
	short t;

	// Kvf[^̌vZ
	if (dwNumOfBytes % 64 == 0) {
		dwNeededSize = dwNumOfBytes + 64;
	}
	else if ((dwNumOfBytes % 64 < 56)) {
		dwNeededSize = (DWORD)floor((double)dwNumOfBytes / 64) * 64 + 64;
	}
	else {
		dwNeededSize = (DWORD)floor((double)dwNumOfBytes / 64) * 64 + 128;
	}
	dwNumOfBlocks = dwNeededSize / 64;
	dwLowNumOfBits = dwNumOfBytes << 3;
	dwHighNumOfBits = dwNumOfBytes >> (8 * sizeof(DWORD) - 3);

	// SHA256̌vZɎgpf[^̐
	pucTempData = (unsigned char*) malloc(dwNeededSize);
	if (pucTempData == 0) {
		return false;
	}
	FillMemory(pucTempData, dwNeededSize, 0);
	memcpy(pucTempData, pucData, dwNumOfBytes);
	pucTempData[dwNumOfBytes] = 0x80;
	for (i = 0; i < 4; i++) {
		pucTempData[dwNeededSize - 8 + i] = (unsigned char)((dwHighNumOfBits >> (24 - i * 8)) & 0xFF);
	}
	for (i = 0; i < 4; i++) {
		pucTempData[dwNeededSize - 4 + i] = (unsigned char)((dwLowNumOfBits >> (24 - i * 8)) & 0xFF);
	}

	// 
	H0 = 0x6a09e667;
	H1 = 0xbb67ae85;
	H2 = 0x3c6ef372;
	H3 = 0xa54ff53a;
	H4 = 0x510e527f;
	H5 = 0x9b05688c;
	H6 = 0x1f83d9ab;
	H7 = 0x5be0cd19;

	// SHA-256̌vZ
	for (i = 0; i < dwNumOfBlocks; i++) {
		
		for (t = 0; t < 16; t++) {
			W[t] = (pucTempData[i * 64 + t * 4] << 24) | (pucTempData[i * 64 + t * 4 + 1] << 16) | (pucTempData[i * 64 + t * 4 + 2] << 8) | pucTempData[i * 64 + t * 4 + 3];
		}

		for (t = 16; t < 64; t++) {
			W[t] = SSigma1(W[t - 2]) + W[t - 7] + SSigma0(W[t - 15]) + W[t - 16];
		}

		a = H0;
		b = H1;
		c = H2;
		d = H3;
		e = H4;
		f = H5;
		g = H6;
		h = H7;

		for (t = 0; t < 64; t++) {
			T1 = h + LSigma1(e) + Ch(e, f, g) + K[t] + W[t];
			T2 = LSigma0(a) + Maj(a, b, c);
			h = g;
			g = f;
			f = e;
			e = d + T1;
			d = c;
			c = b;
			b = a;
			a = T1 + T2;
		}
		
		H0 += a;
		H1 += b;
		H2 += c;
		H3 += d;
		H4 += e;
		H5 += f;
		H6 += g;
		H7 += h;
	}

	// ʂ̏o
	pulResults[0] = H0;
	pulResults[1] = H1;
	pulResults[2] = H2;
	pulResults[3] = H3;
	pulResults[4] = H4;
	pulResults[5] = H5;
	pulResults[6] = H6;
	pulResults[7] = H7;

	// q[v̈̉
	free(pucTempData);

	// ߂l
	return true;
}

//----------------------------------------
//
//  SHA-256̃tH[}bg
//
//   [p[^]
//    (unsigned long*) pulResults: nbVf[^(32oCg)
//    (char*) pcSHA256 : ϊʂۑobt@
//
//   [߂l]
//    (Ȃ)
//----------------------------------------
void SHA256::FormatSHA256(const unsigned long *pulResults, char *pcSHA256) const
{
	ConvDecToHex(pulResults[0], pcSHA256);
	ConvDecToHex(pulResults[1], pcSHA256 + 8);
	ConvDecToHex(pulResults[2], pcSHA256 + 16);
	ConvDecToHex(pulResults[3], pcSHA256 + 24);
	ConvDecToHex(pulResults[4], pcSHA256 + 32);
	ConvDecToHex(pulResults[5], pcSHA256 + 40);
	ConvDecToHex(pulResults[6], pcSHA256 + 48);
	ConvDecToHex(pulResults[7], pcSHA256 + 56);
}


