/*********************************************************************************

 XMLNX


 Copyright (c) 2006-2013, Isao MORI (Orios)
 All rights reserved.

 Redistribution and use in source and binary forms, with or without modification,
 are permitted provided that the following conditions are met:
   * Redistributions of source code must retain the above copyright notice, this
     list of conditions and the following disclaimer.
   * Redistributions in binary form must reproduce the above copyright notice,
     this list of conditions and the following disclaimer in the documentation
     and/or other materials provided with the distribution.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*********************************************************************************/

#include "xml.h"

//==================================================
// RXgN^
//==================================================
XMLElm::XMLElm(int _nLevel)
{	
	nLevel = _nLevel;
	
	nNumOfAttr = 0;
	nNumOfElms = 0;
	nContentLen = 0;
	psAttr = (XML_ATTR*)malloc(1);
	ppoElms = (XMLElm**)malloc(1);
	pcElmName = (char*)malloc(1);
	pcElmName[0] = NULL;
	pcContent = (char*)malloc(1);
	pcContent[0] = NULL;
	bEmpty = false;
	bDeleted = false;
}

//==================================================
// fXgN^
//==================================================
XMLElm::~XMLElm()
{
	int i;
	
	free(pcContent);
	free(pcElmName);
	for (i = 0; i < nNumOfAttr; i++) {
		free(psAttr[i].pcAttrName);
		free(psAttr[i].pcAttrValue);
	}
	free(psAttr);
	for (i = 0; i < nNumOfElms; i++) {
		delete ppoElms[i];
	}
	free(ppoElms);
}



//==================================================
// SXLbv
//==================================================
void XML::SkipS(unsigned char* pucXMLData, DWORD& dwCurPos)
{
	while (
		(pucXMLData[dwCurPos] == 0x20) ||
		(pucXMLData[dwCurPos] == 0x0D) ||
		(pucXMLData[dwCurPos] == 0x0A) ||
		(pucXMLData[dwCurPos] == 0x09)
	) {
		dwCurPos++;
	}
}

//==================================================
// mF
//==================================================
bool XML::CheckStr(char* pcStr, unsigned char* pucXMLData, DWORD& dwCurPos)
{
	int nStrLen = lstrlen((LPCSTR)pcStr);
	int i = 0;
	
	while (i < nStrLen) {
		if (pucXMLData[dwCurPos + i] != pcStr[i]) {
			return false;
		}
		i++;
	}
	dwCurPos += i;
	return true;
}

//==================================================
// GeBeBID擾
//==================================================
XML_ENTITY* XML::psEntities = NULL;
int XML::nNumOfEntities = 0;
int XML::nNumOfObjects = 0;

int XML::GetEntityID(const char* pcEntityName)
{
	int i;

	for (i = 0; i < nNumOfEntities; i++) {
		if (lstrcmp(pcEntityName, psEntities[i].pcEntityName) == 0) {
			return i;
		}
	}
	return -1;
}

//==================================================
// XMLt@Cǂݍ
//==================================================
bool XML::ReadXML(const char* pcFilePath)
{
	HANDLE hFile;
	DWORD dwFileSize;
	DWORD dw;
	
	hFile = CreateFile(pcFilePath, GENERIC_READ, 0, NULL, OPEN_EXISTING, FILE_FLAG_SEQUENTIAL_SCAN, NULL);
	if (hFile != INVALID_HANDLE_VALUE) {
		dwFileSize = GetFileSize(hFile, NULL);
		pucXMLData = (unsigned char*) malloc(dwFileSize + 2);
		ReadFile(hFile, pucXMLData, dwFileSize, &dw, NULL);
		pucXMLData[dwFileSize] = pucXMLData[dwFileSize + 1] = NULL;
		CloseHandle(hFile);
		return true;
		
	}
	else {
		return false;
	}
}

//==================================================
// XMLt@Cۑ
//==================================================
void XML::Save(const char* pcFilePath, bool bOverWrite)
{
	HANDLE hFile;
	DWORD dw;
	
	hFile = CreateFile(pcFilePath, GENERIC_WRITE, 0, NULL, CREATE_NEW, FILE_FLAG_SEQUENTIAL_SCAN, NULL);
	if (bOverWrite && (hFile == INVALID_HANDLE_VALUE)) {
		hFile = CreateFile(pcFilePath, GENERIC_WRITE, 0, NULL, OPEN_EXISTING, FILE_FLAG_SEQUENTIAL_SCAN, NULL);
	}
	if (hFile == INVALID_HANDLE_VALUE) {
		return;
	}
	SetEndOfFile(hFile);
	WriteFile(hFile, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\r\n", 40, &dw, NULL);
	poRoot->WriteElms(hFile);
	WriteFile(hFile, "\r\n", 2, &dw, NULL);
	CloseHandle(hFile);
}

//==================================================
// prologǂݍ
//==================================================
void XML::ReadProlog()
{
	while (
		(pucXMLData[dwCurPos] != 0x3C) ||
		(pucXMLData[dwCurPos + 1] == 0x3F) || (pucXMLData[dwCurPos + 1] == 0x21)
	) {

		SkipS(pucXMLData, dwCurPos);
		
		// XMLDecl & PI ()
		if (
			(pucXMLData[dwCurPos] == 0x3C) &&
			(pucXMLData[dwCurPos + 1] == 0x3F)
		) {
			dwCurPos += 3;
			while (
				(pucXMLData[dwCurPos - 1] != 0x3F) ||
				(pucXMLData[dwCurPos] != 0x3E)
			) {
				dwCurPos++;
			}

		// "<!"n܂ꍇ
		}
		else if (
			(pucXMLData[dwCurPos] == 0x3C) &&
			(pucXMLData[dwCurPos + 1] == 0x21)
		) {
			
			dwCurPos += 2;
			// Rg()
			if (
				(pucXMLData[dwCurPos] == 0x2D) &&
				(pucXMLData[dwCurPos + 1] == 0x2D)
			) {
				dwCurPos += 4;
				while (
					(pucXMLData[dwCurPos - 2] == 0x2D) &&
					(pucXMLData[dwCurPos - 1] == 0x2D) &&
					(pucXMLData[dwCurPos] == 0x3E)
				) {
					dwCurPos++;
				}
				
			// DOCTYPE
			}
			else if ((!CheckStr("DOCTYPE", pucXMLData, dwCurPos))) {

				while (pucXMLData[dwCurPos] != 0x3E) {
					if (pucXMLData[dwCurPos] == 0x5B) {
//						ReadDTD(pucXMLData, dwCurPos, -1);
					}
					dwCurPos++;
				}
			}
		}
		else if ((pucXMLData[dwCurPos] == 0x3C)) {
			return;
		}
		dwCurPos++;
	}

}

//==================================================
// DTDǂݍ
//==================================================
void XML::ReadDTD(char* pcDTDData, DWORD& dwDTDCurPos, int nEndPos)
{
	while (true) {
		if (dwDTDCurPos == nEndPos) {
			return;
		}
		if (!CheckStr("<!ENTITY", pucXMLData, dwCurPos)) {
			SkipS(pucXMLData, dwCurPos);
		}
		dwCurPos++;
	}
}

//==================================================
// RXgN^
//==================================================
XML::XML()
{
	pucXMLData = NULL;

	if (nNumOfObjects == 0) {
		psEntities = (XML_ENTITY*)malloc(5 * sizeof(XML_ENTITY));
	
		psEntities[0].pcEntityName = (char*)malloc(3);
		lstrcpy((LPSTR)psEntities[0].pcEntityName, "lt");
		psEntities[0].pcEntityValue = (char*)malloc(2);
		lstrcpy((LPSTR)psEntities[0].pcEntityValue, "<");
	
		psEntities[1].pcEntityName = (char*)malloc(3);
		lstrcpy((LPSTR)psEntities[1].pcEntityName, "gt");
		psEntities[1].pcEntityValue = (char*)malloc(2);
		lstrcpy((LPSTR)psEntities[1].pcEntityValue, ">");
	
		psEntities[2].pcEntityName = (char*)malloc(4);
		lstrcpy((LPSTR)psEntities[2].pcEntityName, "amp");
		psEntities[2].pcEntityValue = (char*)malloc(2);
		lstrcpy((LPSTR)psEntities[2].pcEntityValue, "&");
	
		psEntities[3].pcEntityName = (char*)malloc(5);
		lstrcpy((LPSTR)psEntities[3].pcEntityName, "apos");
		psEntities[3].pcEntityValue = (char*)malloc(2);
		lstrcpy((LPSTR)psEntities[3].pcEntityValue, "'");
	
		psEntities[4].pcEntityName = (char*)malloc(5);
		lstrcpy((LPSTR)psEntities[4].pcEntityName, "quot");
		psEntities[4].pcEntityValue = (char*)malloc(2);
		lstrcpy((LPSTR)psEntities[4].pcEntityValue, "\"");

		nNumOfEntities = 5;
	}
	
	nNumOfObjects++;
	poRoot = new XMLElm(0);
}

//==================================================
// fXgN^
//==================================================
XML::~XML()
{
	int i;
	
	nNumOfObjects--;
	if (nNumOfObjects == 0) {
		for (i = 0; i < nNumOfEntities; i++) {
			free(psEntities[i].pcEntityName);
			free(psEntities[i].pcEntityValue);
		}
		free(psEntities);
	}
	
	if (pucXMLData != NULL) {
		free(pucXMLData);
	}
	delete poRoot;
}

//==================================================
// XMLǂݍ
//==================================================
void XML::Load(const char* pcFilePath)
{
	CharSet *poCharSet;
	int nNeededSize;
	
	if (!ReadXML(pcFilePath)) {
		return;
	}
	
	// UTF-16 -> UTF-8
	if ((pucXMLData[1] == 0) || ((pucXMLData[0] == 0xFF) && (pucXMLData[1] == 0xFE))) {
		poCharSet = new CharSet();
		nNeededSize = poCharSet->ConvertText((unsigned char*)pucXMLData, UCS2_TO_UTF8);
		free(pucXMLData);
		pucXMLData = (unsigned char*)malloc(nNeededSize);
		poCharSet->CopyText((unsigned char*)pucXMLData);
		delete poCharSet;
	}
	if (
		(pucXMLData[0] == 0xEF) &&
		(pucXMLData[1] == 0xBB) &&
		(pucXMLData[2] == 0xBF)
	) {
		dwCurPos = 3;
	}
	else {
		dwCurPos = 0;
	}
	
	// prolog
	ReadProlog();
	
	// Element
	poRoot->ReadElms(pucXMLData, dwCurPos, false, false);
}

//==================================================
// vfǂݍ
//==================================================
void XMLElm::ReadElms(unsigned char* pucXMLData, DWORD& dwCurPos, bool bOnlyContent, bool bCDATA)
{
	int i = 0;
	int nDataLen = lstrlen((LPCSTR)pucXMLData);
	int nAttrValueLen;
	char cCiteChar;
	int nEntityID;
	char* pcEntityName;
	char* pcCharRef;
	int nCharCode;
	char* pcStop;

	if (!bOnlyContent) {

		if (pucXMLData[dwCurPos] != 0x3C) {
			return;
		}
		dwCurPos++;

		// Name
		while (
			(pucXMLData[dwCurPos + i] != 0x20) &&
			(pucXMLData[dwCurPos + i] != 0x0D) &&
			(pucXMLData[dwCurPos + i] != 0x0A) &&
			(pucXMLData[dwCurPos + i] != 0x09) &&
			(pucXMLData[dwCurPos + i] != 0x3E)
		) {
			i++;
		}
		pcElmName = (char*)realloc(pcElmName, i + 1);
		memcpy(pcElmName, pucXMLData + dwCurPos, i);
		pcElmName[i] = 0;
		dwCurPos += i;

		// Attributes
		while (true) {

			XML::SkipS(pucXMLData, dwCurPos);
			if (pucXMLData[dwCurPos] == 0x3E) {
				dwCurPos++;
				break;
			}
			else if (
				(pucXMLData[dwCurPos] == 0x2F) &&
				(pucXMLData[dwCurPos + 1] == 0x3E)
			) {
				dwCurPos += 2;
				bEmpty = true;
				break;
			}

			// Name
			i = 0;
			while (
				(pucXMLData[dwCurPos + i] != 0x20) &&
				(pucXMLData[dwCurPos + i] != 0x0D) &&
				(pucXMLData[dwCurPos + i] != 0x0A) &&
				(pucXMLData[dwCurPos + i] != 0x09) &&
				(pucXMLData[dwCurPos + i] != 0x3D)
			) {
				i++;
			}
			nNumOfAttr++;
			psAttr = (XML_ATTR*)realloc(psAttr, nNumOfAttr * sizeof(XML_ATTR));
			psAttr[nNumOfAttr - 1].pcAttrName = (char*)malloc(i + 1);
			memcpy(psAttr[nNumOfAttr - 1].pcAttrName, pucXMLData + dwCurPos, i);
			psAttr[nNumOfAttr - 1].pcAttrName[i] = NULL;
			dwCurPos += i;

			// AttValue
			while (
				(pucXMLData[dwCurPos] == 0x20) ||
				(pucXMLData[dwCurPos] == 0x3D)
			) {
				dwCurPos++;
			}
			if ((pucXMLData[dwCurPos] == 0x22) || (pucXMLData[dwCurPos] == 0x27)) {

				cCiteChar = pucXMLData[dwCurPos];
				dwCurPos++;
				i = 0;
				nAttrValueLen = 0;
				psAttr[nNumOfAttr - 1].pcAttrValue = (char*)malloc(1);
				while (pucXMLData[dwCurPos + i] != cCiteChar) {

					if (pucXMLData[dwCurPos + i] == 0x26) {

						psAttr[nNumOfAttr - 1].pcAttrValue = (char*)realloc(psAttr[nNumOfAttr - 1].pcAttrValue, nAttrValueLen + i);
						memcpy(psAttr[nNumOfAttr - 1].pcAttrValue + nAttrValueLen, pucXMLData + dwCurPos, i);
						nAttrValueLen += i;
						dwCurPos += i + 1;
						i = 0;

						while (pucXMLData[dwCurPos + i] != 0x3B) {
							i++;
						}
						pcEntityName = (char*)malloc(i + 1);
						memcpy(pcEntityName, pucXMLData + dwCurPos, i);
						pcEntityName[i] = NULL;

						if (pcEntityName[0] == 0x23) {
							if (pcEntityName[1] == 0x78) {
								pcCharRef = (char*)malloc(lstrlen((LPCSTR)pcEntityName) - 1);
								lstrcpy((LPSTR)pcCharRef, pcEntityName + 2);
								nCharCode = strtol(pcCharRef, &pcStop, 16);
								free(pcCharRef);

							}
							else {
								pcCharRef = (char*)malloc(lstrlen((LPCSTR)pcEntityName));
								lstrcpy((LPSTR)pcCharRef, pcEntityName + 1);
								nCharCode = strtol(pcCharRef, &pcStop, 10);
								free(pcCharRef);
							}

							psAttr[nNumOfAttr - 1].pcAttrValue = (char*)realloc(psAttr[nNumOfAttr - 1].pcAttrValue, nAttrValueLen + 1);
							psAttr[nNumOfAttr - 1].pcAttrValue[nAttrValueLen] = nCharCode;
							nAttrValueLen++;

						}
						else {
							nEntityID = XML::GetEntityID(pcEntityName);
							dwCurPos += i + 1;
							i = 0;
							free(pcEntityName);

							if (nEntityID != -1) {
								psAttr[nNumOfAttr - 1].pcAttrValue = (char*)realloc(psAttr[nNumOfAttr - 1].pcAttrValue, nAttrValueLen + lstrlen((LPCSTR)XML::psEntities[nEntityID].pcEntityValue) + 1);
								lstrcpy((LPSTR)psAttr[nNumOfAttr - 1].pcAttrValue + nAttrValueLen, XML::psEntities[nEntityID].pcEntityValue);
								nAttrValueLen += lstrlen((LPCSTR)XML::psEntities[nEntityID].pcEntityValue);
							}
						}

						dwCurPos += i + 1;
						i = 0;
					}
					else {
						i++;
					}
				}
				psAttr[nNumOfAttr - 1].pcAttrValue = (char*)realloc(psAttr[nNumOfAttr - 1].pcAttrValue, nAttrValueLen + i + 1);
				memcpy(psAttr[nNumOfAttr - 1].pcAttrValue + nAttrValueLen, pucXMLData + dwCurPos, i);
				psAttr[nNumOfAttr - 1].pcAttrValue[nAttrValueLen + i] = NULL;
				dwCurPos += i + 1;

			}
			else {
				return;
			}

		}
	}

	// Content
	if (!bEmpty) {
		i = 0;
		while ((dwCurPos + i) < (DWORD)nDataLen) {
			if (!bCDATA && (pucXMLData[dwCurPos + i] == 0x3C) && (pucXMLData[dwCurPos + i + 1] != 0x3F)) {

				pcContent = (char*)realloc(pcContent, nContentLen + i + 1);
				memcpy(pcContent + nContentLen, pucXMLData + dwCurPos, i);
				nContentLen += i;
				pcContent[nContentLen] = NULL;

				dwCurPos += i + 2;
				if ((pucXMLData[dwCurPos - 1] == 0x2F) && XML::CheckStr(pcElmName, pucXMLData, dwCurPos)) {
					dwCurPos++;
					return;

				}
				else if (
					(pucXMLData[dwCurPos - 1] == 0x21) &&
					(pucXMLData[dwCurPos    ] == 0x2D) &&
					(pucXMLData[dwCurPos + 1] == 0x2D)
				) {
					dwCurPos += 2;
					while (
						(pucXMLData[dwCurPos    ] != 0x2D) ||
						(pucXMLData[dwCurPos + 1] != 0x2D) ||
						(pucXMLData[dwCurPos + 2] != 0x3E)
					) {
						dwCurPos++;
					}
					dwCurPos += 3;

				}
				else if (((pucXMLData[dwCurPos - 1] == 0x21) && XML::CheckStr("[CDATA[", pucXMLData, dwCurPos))) {
					
					i = 0;
					while (
						(pucXMLData[dwCurPos + i] != 0x5D) ||
						(pucXMLData[dwCurPos + i + 1] != 0x5D) ||
						(pucXMLData[dwCurPos + i + 2] != 0x3E)
					) {
						i++;
					}
					pcContent = (char*)realloc(pcContent, nContentLen + i + 1);
					memcpy(pcContent + nContentLen, pucXMLData + dwCurPos, i);
					nContentLen += i;
					pcContent[nContentLen] = NULL;
					dwCurPos += i + 3;

				}
				else {
					dwCurPos -= 2;
					nNumOfElms++;
					ppoElms = (XMLElm**)realloc(ppoElms, nNumOfElms * sizeof(XMLElm*));
					ppoElms[nNumOfElms - 1] = new XMLElm(nLevel + 1);
					ppoElms[nNumOfElms - 1]->ReadElms(pucXMLData, dwCurPos, false, false);
					pcContent = (char*)realloc(pcContent, nContentLen + 1);
					pcContent[nContentLen] = NULL;
					nContentLen++;
				}
				
				i = 0;

			}
			else if ((!bCDATA && (pucXMLData[dwCurPos + i] == 0x26))) {
				
				pcContent = (char*)realloc(pcContent, nContentLen + i);
				memcpy(pcContent + nContentLen, pucXMLData + dwCurPos, i);
				nContentLen += i;
				
				dwCurPos += i + 1;
				i = 0;

				while (pucXMLData[dwCurPos + i] != 0x3B) {
					i++;
				}
				pcEntityName = (char*)malloc(i + 1);
				memcpy(pcEntityName, pucXMLData + dwCurPos, i);
				pcEntityName[i] = NULL;

				if (pcEntityName[0] == 0x23) {
					if (pcEntityName[1] == 0x78) {
						pcCharRef = (char*)malloc(lstrlen((LPCSTR)pcEntityName) - 1);
						lstrcpy((LPSTR)pcCharRef, pcEntityName + 2);
						nCharCode = strtol(pcCharRef, &pcStop, 16);
						free(pcCharRef);

					}
					else {
						pcCharRef = (char*)malloc(lstrlen((LPCSTR)pcEntityName));
						lstrcpy((LPSTR)pcCharRef, pcEntityName + 1);
						nCharCode = strtol(pcCharRef, &pcStop, 10);
						free(pcCharRef);
					}

					pcContent = (char*)realloc(pcContent, nContentLen + 1);
					pcContent[nContentLen] = nCharCode;
					nContentLen++;

				}
				else {
					nEntityID = XML::GetEntityID(pcEntityName);

					if (nEntityID != -1) {
						pcContent = (char*)realloc(pcContent, nContentLen + lstrlen((LPCSTR)XML::psEntities[nEntityID].pcEntityValue) + 1);
						lstrcpy((LPSTR)pcContent + nContentLen, XML::psEntities[nEntityID].pcEntityValue);
						nContentLen += lstrlen((LPCSTR)XML::psEntities[nEntityID].pcEntityValue);
					}
				}

				dwCurPos += i + 1;
				i = 0;
				free(pcEntityName);
				
			}
			else {
				i++;
			}
		}

		pcContent = (char*)realloc(pcContent, nContentLen + i + 1);
		memcpy(pcContent + nContentLen, pucXMLData + dwCurPos, i);
		nContentLen += i;
		pcContent[nContentLen] = NULL;
	}

}

//==================================================
// vf
//==================================================
void XMLElm::WriteElms(HANDLE hFile)
{
	int i;
	int j;
	int nStartPos;
	int nElmNum = 0;
	int nAttrValueLen;
	DWORD dw;
	
	if (bDeleted) {
		return;
	}
	
	WriteFile(hFile, "<", 1, &dw, NULL);
	WriteFile(hFile, pcElmName, lstrlen((LPCSTR)pcElmName), &dw, NULL);
	for (j = 0; j < nNumOfAttr; j++) {
		WriteFile(hFile, " ", 1, &dw, NULL);
		WriteFile(hFile, psAttr[j].pcAttrName, lstrlen((LPCSTR)psAttr[j].pcAttrName), &dw, NULL);
		WriteFile(hFile, "=\"", 2, &dw, NULL);

		nStartPos = 0;
		i = 0;
		nAttrValueLen = lstrlen((LPCSTR)psAttr[j].pcAttrValue);
		while ((nStartPos + i) < nAttrValueLen) {
			if (psAttr[j].pcAttrValue[nStartPos + i] == 0x3C) {
				WriteFile(hFile, psAttr[j].pcAttrValue + nStartPos, i, &dw, NULL);
				nStartPos += i + 1;
				i = 0;
				
				WriteFile(hFile, "&lt;", 4, &dw, NULL);

			}
			else if ((psAttr[j].pcAttrValue[nStartPos + i] == 0x3E)) {
				WriteFile(hFile, psAttr[j].pcAttrValue + nStartPos, i, &dw, NULL);
				nStartPos += i + 1;
				i = 0;
				
				WriteFile(hFile, "&gt;", 4, &dw, NULL);

			}
			else if ((psAttr[j].pcAttrValue[nStartPos + i] == 0x26)) {
				WriteFile(hFile, psAttr[j].pcAttrValue + nStartPos, i, &dw, NULL);
				nStartPos += i + 1;
				i = 0;
				
				WriteFile(hFile, "&amp;", 5, &dw, NULL);

			}
			else if ((psAttr[j].pcAttrValue[nStartPos + i] == 0x22)) {
				WriteFile(hFile, psAttr[j].pcAttrValue + nStartPos, i, &dw, NULL);
				nStartPos += i + 1;
				i = 0;
				
				WriteFile(hFile, "&quot;", 6, &dw, NULL);

			}
			else {
				i++;
			}
		}
		if (i != 0) {
			WriteFile(hFile, psAttr[j].pcAttrValue + nStartPos, i, &dw, NULL);
		}
		WriteFile(hFile, "\"", 1, &dw, NULL);
	}
	
	if (bEmpty) {
		WriteFile(hFile, " />", 3, &dw, NULL);

	}
	else {
		WriteFile(hFile, ">", 1, &dw, NULL);
		
		nStartPos = 0;
		i = 0;
		while ((nStartPos + i) < nContentLen) {
			if (pcContent[nStartPos + i] == NULL) {
				
				WriteFile(hFile, pcContent + nStartPos, i, &dw, NULL);
				nStartPos += i + 1;
				i = 0;
	
				ppoElms[nElmNum]->WriteElms(hFile);
				nElmNum++;
	
			}
			else if ((pcContent[nStartPos + i] == 0x3C)) {
				WriteFile(hFile, pcContent + nStartPos, i, &dw, NULL);
				nStartPos += i + 1;
				i = 0;
				
				WriteFile(hFile, "&lt;", 4, &dw, NULL);

			}
			else if ((pcContent[nStartPos + i] == 0x3E)) {
				WriteFile(hFile, pcContent + nStartPos, i, &dw, NULL);
				nStartPos += i + 1;
				i = 0;
				
				WriteFile(hFile, "&gt;", 4, &dw, NULL);

			}
			else if ((pcContent[nStartPos + i] == 0x26)) {
				WriteFile(hFile, pcContent + nStartPos, i, &dw, NULL);
				nStartPos += i + 1;
				i = 0;
				
				WriteFile(hFile, "&amp;", 5, &dw, NULL);

			}
			else {
				i++;
			}
		}
		
		if (i != 0) {
			WriteFile(hFile, pcContent + nStartPos, i, &dw, NULL);
		}
		
		WriteFile(hFile, "</", 2, &dw, NULL);
		WriteFile(hFile, pcElmName, lstrlen((LPCSTR)pcElmName), &dw, NULL);
		WriteFile(hFile, ">", 1, &dw, NULL);
	}
}

//==================================================
// vfǉ(A^)
//==================================================
int XMLElm::AddElm(char* pcNewElmData, bool bCDATA)
{
	DWORD dw = 0;
	int i;
	
	nNumOfElms++;
	ppoElms = (XMLElm**)realloc(ppoElms, nNumOfElms * sizeof(XMLElm*));
	ppoElms[nNumOfElms - 1] = new XMLElm(nLevel + 1);
	ppoElms[nNumOfElms - 1]->ReadElms((unsigned char*)pcNewElmData, dw, false, bCDATA);
	pcContent = (char*)realloc(pcContent, nContentLen + nLevel + 5);
	pcContent[nContentLen] = 0x09;
	pcContent[nContentLen + 1] = NULL;
	pcContent[nContentLen + 2] = 0x0D;
	pcContent[nContentLen + 3] = 0x0A;
	for (i = 0; i < nLevel; i++) {
		pcContent[nContentLen + 4 + i] = 0x09;
	}
	pcContent[nContentLen + 4 + nLevel] = NULL;
	nContentLen += nLevel + 4;
	
	return nNumOfElms - 1;
}

//==================================================
// vfǉ(B^)
//==================================================
int XMLElm::AddElm(char* pcNewElmName, char* pcNewElmContent, bool bCDATA)
{
	DWORD dw = 0;
	int i;
	
	nNumOfElms++;
	ppoElms = (XMLElm**)realloc(ppoElms, nNumOfElms * sizeof(XMLElm*));
	ppoElms[nNumOfElms - 1] = new XMLElm(nLevel + 1);
	ppoElms[nNumOfElms - 1]->SetElmName(pcNewElmName);
	ppoElms[nNumOfElms - 1]->ReadElms((unsigned char*)pcNewElmContent, dw, true, bCDATA);
	pcContent = (char*)realloc(pcContent, nContentLen + nLevel + 5);
	pcContent[nContentLen] = 0x09;
	pcContent[nContentLen + 1] = NULL;
	pcContent[nContentLen + 2] = 0x0D;
	pcContent[nContentLen + 3] = 0x0A;
	for (i = 0; i < nLevel; i++) {
		pcContent[nContentLen + 4 + i] = 0x09;
	}
	pcContent[nContentLen + 4 + nLevel] = NULL;
	nContentLen += nLevel + 4;
	
	return nNumOfElms - 1;
}

//==================================================
// vfݒ
//==================================================
void XMLElm::SetElmName(char* pcNewElmName)
{
	pcElmName = (char*)realloc(pcElmName, lstrlen((LPCSTR)pcNewElmName) + 1);
	lstrcpy((LPSTR)pcElmName, pcNewElmName);
}

//==================================================
// ǉ
//==================================================
int XMLElm::AddAttr(char* pcNewAttrName, char* pcNewAttrValue)
{
	nNumOfAttr++;
	psAttr = (XML_ATTR*)realloc(psAttr, nNumOfAttr * sizeof(XML_ATTR));
	psAttr[nNumOfAttr - 1].pcAttrName = (char*)malloc(lstrlen((LPCSTR)pcNewAttrName) + 1);
	lstrcpy((LPSTR)psAttr[nNumOfAttr - 1].pcAttrName, pcNewAttrName);
	psAttr[nNumOfAttr - 1].pcAttrValue = (char*)malloc(lstrlen((LPCSTR)pcNewAttrValue) + 1);
	lstrcpy((LPSTR)psAttr[nNumOfAttr - 1].pcAttrValue, pcNewAttrValue);
	
	return nNumOfAttr - 1;
}

//==================================================
// vfID擾
//==================================================
int XMLElm::GetElmID(const char* pcElmName, int nStartID)
{
	int i;

	for (i = nStartID; i < nNumOfElms; i++) {
		if (lstrcmp(pcElmName, ppoElms[i]->pcElmName) == 0) {
			return i;
		}
	}
	return -1;
}

//==================================================
// ID擾
//==================================================
int XMLElm::GetAttrID(const char* pcAttrName)
{
	int i;

	for (i = 0; i < nNumOfAttr; i++) {
		if (lstrcmp(pcAttrName, psAttr[i].pcAttrName) == 0) {
			return i;
		}
	}
	return -1;
}

//==================================================
// eύX
//==================================================
void XMLElm::ChangeContent(char* pcNewContent, bool bCDATA)
{
	int i;
	DWORD dw = 0;
	
	for (i = 0; i < nNumOfElms; i++) {
		delete ppoElms[i];
	}
	nNumOfElms = 0;
	nContentLen = 0;
	ppoElms = (XMLElm**)realloc(ppoElms, 0);
	pcContent = (char*)realloc(pcContent, 1);
	pcContent[0] = NULL;

	if (lstrlen((LPCSTR)pcNewContent) == 0) {
		bEmpty = true;
	}
	else {
		bEmpty = false;
		ReadElms((unsigned char*)pcNewContent, dw, true, bCDATA);
	}
}

//==================================================
// vf폜
//==================================================
void XMLElm::DeleteElm()
{
	int i;
	
	for (i = 0; i < nNumOfElms; i++) {
		delete ppoElms[i];
	}
	nNumOfElms = 0;
	nContentLen = 0;
	ppoElms = (XMLElm**)realloc(ppoElms, 0);
	pcContent = (char*)realloc(pcContent, 1);
	pcContent[0] = NULL;
	bDeleted = true;
}

//==================================================
// lύX
//==================================================
void XMLElm::ChangeAttrValue(int nAttrID, char* pcNewAttrValue)
{
	psAttr[nAttrID].pcAttrValue = (char*)malloc(lstrlen((LPCSTR)pcNewAttrValue) + 1);
	lstrcpy((LPSTR)psAttr[nAttrID].pcAttrValue, pcNewAttrValue);

	return;
}
