/*********************************************************************************

 XMLNX


 Copyright (c) 2006-2013, Isao MORI (Orios)
 All rights reserved.

 Redistribution and use in source and binary forms, with or without modification,
 are permitted provided that the following conditions are met:
   * Redistributions of source code must retain the above copyright notice, this
     list of conditions and the following disclaimer.
   * Redistributions in binary form must reproduce the above copyright notice,
     this list of conditions and the following disclaimer in the documentation
     and/or other materials provided with the distribution.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*********************************************************************************/

#ifndef XML_HEADER
#define XML_HEADER

#include <windows.h>
#include "CharSet.h"

//==================================================
// \̂̐錾
//==================================================
struct XML_ATTR {
	char* pcAttrName;
	char* pcAttrValue;
};

struct XML_ENTITY {
	char* pcEntityName;
	char* pcEntityValue;
};

//==================================================
// XMLElmNX
//==================================================
class XMLElm {
public:
	int nLevel;
	char* pcElmName;
	int nNumOfAttr;
	XML_ATTR* psAttr;
	XMLElm** ppoElms;
	int nNumOfElms;
	char* pcContent;
	int nContentLen;
	bool bEmpty;
	bool bDeleted;

	XMLElm(int _nLevel);
	~XMLElm();

	void ReadElms(unsigned char* pucXMLData, DWORD& dwCurPos, bool bOnlyContent, bool bCDATA);
	void WriteElms(HANDLE hFile);
	void SetElmName(char* pcNewElmName);
	int AddElm(char* pcNewElmData, bool bCDATA);
	int AddElm(char* pcNewElmName, char* pcNewElmContent, bool bCDATA);
	int AddAttr(char* pcNewAttrName, char* pcNewAttrValue);
	void ChangeContent(char* pcNewContent, bool bCDATA);
	int GetElmID(const char* pcElmName, int nStartID);
	int GetAttrID(const char* pcAttrName);
	void DeleteElm();
	void ChangeAttrValue(int nAttrID, char* pcNewAttrValue);
};

//==================================================
// XMLNX
//==================================================
class XML {
private:
	bool ReadXML(const char* pcFilePath);
	void ReadProlog();
	void ReadDTD(char* pcDTDData, DWORD& dwDTDCurPos, int nEndPos);
	
	void WriteXML(const char* pcFilePath);

public:
	unsigned char* pucXMLData;
	DWORD dwCurPos;
	XMLElm* poRoot;
	static XML_ENTITY* psEntities;
	static int nNumOfEntities;
	static int nNumOfObjects;

	XML();
	~XML();
	static void SkipS(unsigned char* pucXMLData, DWORD& dwCurPos);
	static bool CheckStr(char* pcStr, unsigned char* pucXMLData, DWORD& dwCurPos);
	static int GetEntityID(const char* pcEntityName);
	void Load(const char* pcFilePath);
	void Save(const char* pcFilePath, bool bOverWrite);

};

#endif
